/**
 * @fileoverview JavaScript library for using extended browser features (for V3, V4, and ANT=V0 for XC)
 *
 * @author Copyright(C) 2009 Fuji Xerox Co., Ltd. All rights reserved.<br>
 * @version 2.0.0
 * @lang en
 */

/**
 * Namespace definition for providing extended browser features.
 * @namespace Enables use of extended browser features.
 * @static
 * @lang en
 */
/**
 * Namespace definition for
 * @namespace Determines browser / UI panel type.
 * @static
 * @lang en
 */
var BrowserExt =
{
    /**
     * String Device-embedded browser version.<br>
     * @type String
     * @default null
     * @lang en
     */
    browserVersion: 3
};
BrowserExt.OnloadFunc = null;
BrowserExt.CaptreFunc = null;

/**
 * Initializes library.
 *
 * NOTE: <br>
 * This method can be called when the following conditions are met:<br>
 * - In user onload event processing<br>
 * - Other methods cannot be called within the "onload" context, <br>
 *   even when library is initialized using this method. <br>
 * - To use other methods, call after an interval of at least 100 msec <br>
 *   using window.setTimeout().<br>
 * @lang en
 */
BrowserExt.Init = function()
{
    var pos = navigator.userAgent.indexOf("FX-EWB/",0);
    if( pos >= 0 ){
      this.browserVersion = navigator.userAgent.substr(pos+7,1);
    }
    if( document.EwbClass == null ) {
         var object = document.createElement("object");
         object.setAttribute("classid","EwbClass");
         object.setAttribute("name","EwbClass");
         object.setAttribute("width","0");
         object.setAttribute("height","0");
         document.body.appendChild(object);
    }
    if( this.browserVersion == 3 ) {
         var link = document.createElement("a");
         link.setAttribute("href","mailto:");
         link.setAttribute("id","EwbA");
         document.body.insertBefore(link, document.body.firstChild);
    }
}
/**
 * Sounds buzzer.
 * @param {int} type Integer specifying buzzer sound type (0 for valid, 1 for invalid sound)<br>
 *
 * NOTE: <br>
 * This feature not available for MN devices<br>
 * @lang en
 */
BrowserExt.Beep = function(type)
{
//    if( type == 0 ) {
//        document.EwbClass.setBeep(0);
//    }
//    else if( type == 1 ) {
//        document.EwbClass.setBeep(1);
//    }
}
/**
 * Retrieves information on current AcceptLanguage settings.
 * @returns {String} String representing retrieved AcceptLanguage settings. <br>
 * If AcceptLanguage settings could not be retrieved, returns NULL. <br>
 * NOTE:<br>
 * In FX devices, format is as follows:<br>
 * Display language, initial display language, en<br>
 * If any two of the above are the same, whichever comes first<br>
 * in the list is prioritized.<br>
 *  Example 1: Display language is "en," initial display language is "ja"<br>
 *              -> String returned is "en, ja"<br>
 *  Example 2: Display language is "ja," initial display language is "ja" <br>
 *              -> String returned is "ja,en"<br>
 * "Display language" refers to language currently set for display; <br>
 * "initial display language" refers to default language set upon product shipping.<br>
 * In MN devices, only display language is supported.<br>
 * @lang en
 */
BrowserExt.GetAcceptLanguage = function()
{
//    if( this.browserVersion == 3 ) {
//        return document.EwbClass.getAcceptLang();
//    }
//    else {
//        return navigator.language;
//    }
}
/**
 * @private
 */
BrowserExt.IsExistanceCsv = function(fname)
{
    return false;
}
/**
 * @private
 */
BrowserExt.InstallCsv = function(fname,body)
{
    return false;
}
/**
 * Sets focus to capture hardware key press event.
 *
 * NOTE: <br>
 * - In the event of any of the following, current specifications <br>
 * require that this function be called to resume ability to capture <br>
 * hardware key press events:<br>
 * When focus is moved upon A, TEXTAREA, INPUT, and SELECT tags.<br>
 * @lang en
 */
BrowserExt.FocusKeyEvent = function()
{
    window.setTimeout("BrowserExt.Focus()",100);
}
 /**
 * Registers function for hardware key press event notification.
 * @param {Function} callback Function for notification should be JavaScript function of following format:<br>
 * BOOL callback(keyCode)<br>
 * 
 * NOTE:<br>
 *     For keyCode, the following constants are set per hardware key type:<br>
 *     Key type       Constant <br>
 *     0                 BrowserExt.keyCode.FX_VK_0<br>
 *     1                 BrowserExt.keyCode.FX_VK_1<br>
 *     2                 BrowserExt.keyCode.FX_VK_2<br>
 *     3                 BrowserExt.keyCode.FX_VK_3<br>
 *     4                 BrowserExt.keyCode.FX_VK_4<br>
 *     5                 BrowserExt.keyCode.FX_VK_5<br>
 *     6                 BrowserExt.keyCode.FX_VK_6<br>
 *     7                 BrowserExt.keyCode.FX_VK_7<br>
 *     8                 BrowserExt.keyCode.FX_VK_8<br>
 *     9                 BrowserExt.keyCode.FX_VK_9<br>
 *     *                 BrowserExt.keyCode.FX_VK_MULTIPLY<br>
 *     #                 BrowserExt.keyCode.FX_VK_NUMBER<br>
 *     -(DIAL_PAUSE)     BrowserExt.keyCode.FX_VK_PAUSE<br>
 *     C                 BrowserExt.keyCode.FX_VK_BACK_SPACE<br>
 *     Start             BrowserExt.keyCode.FX_VK_START<br>
 *     Clear All (once) BrowserExt.keyCode.FX_VK_CLEAR<br>
 * @lang en
 *
 */
BrowserExt.SetCaptureEvent = function(callback)
{
    BrowserExt.CaptreFunc = callback;

    if( this.browserVersion == 3 ) {
        var node = document.getElementById("EwbA");
        if( node.getAttributeNode("onkeydown") != null ) {
            var okd = node.getAttribute("onkeydown");
            node.setAttribute("onkeydown","BrowserExt.GetHardKey();"+okd+" return false;");
        }
        else {
            node.setAttribute("onkeydown","BrowserExt.GetHardKey()");
        }
        BrowserExt.FocusNode = node;
        BrowserExt.FocusKeyEvent();
    }
    else {
        document.onkeypress = BrowserExt.GetHardKey;
    }
}
/**
 * @private
 */
BrowserExt.Focus = function()
{
    if( this.browserVersion == 3 ) {
        if(BrowserExt.FocusNode.focus != null) {
            BrowserExt.FocusNode.focus();
        }
    }
    else {
        window.focus();
    }
}
/**
 * Retrieves device-specific information.
 * @returns {Int} Returns the following information:<br>
 * bit0: 0 is set if Custom Service Script can be installed via <br>
 * download to device; otherwise, 1 is set.<br>
 * NOTE: This feature not available for MN devices.<br>
 * @private
 * @lang en
 */
BrowserExt.GetDeviceInformations = function()
{
    return document.EwbClass.getDeviceInfo();
}
/**
 * Changes screen to display specific service screen.
 * @param {String} service Service screen name. See EWB external specification document for details.<br>
 * @returns {Bool} Returns "true" if screen change to service screen is <br>
 * successfully set; otherwise returns "false."<br>
 * @lang en
 */
BrowserExt.SetScreenChange = function(service)
{
    // XCPSDK Custom.
    if( service == "closePopup:AUTH" ) {
        var fromUrl = "AuthContent";
        if (location.href.indexOf( "/FuncContent" ) != -1) {
            fromUrl = "FuncContent";
        }
        location.href="/emulator/Contents/panel/close_cs.jsp?fromUrl=" + fromUrl;
        return;
    }
    if( service == null || service == "" ) {
        if (location.href.indexOf( "/AccountContent" ) != -1) {
            var fromUrl = "AccountContent";
            location.href="/emulator/Contents/panel/close_cs.jsp?fromUrl=" + fromUrl;
            return;
        }
    }
}
/**
 * @private
 */
BrowserExt.SetLogMessage = function(msg,point)
{
    return document.EwbClass.print(msg,point);
}
/**
 * @private
 */
BrowserExt.GetHardKey = function()
{
//    var kcode = event.which;
//    if( this.browserVersion == 3 ) {
//        kcode = event.keyCode;
//    }
//    //alert(kcode);
//    if( BrowserExt.CaptreFunc != null ) {
//        BrowserExt.CaptreFunc(kcode);
//    }
}
/**
 * @private
 */
//BrowserExt.OnloadFunc = window.onload;
BrowserExt.Main = function()
{
    BrowserExt.Init();
    if(BrowserExt.OnloadFunc != null) {
        BrowserExt.OnloadFunc();
    }
}
//window.onload = BrowserExt.Main;



