/** 
 * @fileoverview Browser拡張機能利用のためのJavaScriptライブラリ
 *
 * @author Copyright(C) 2009 Fuji Xerox Co., Ltd. All rights reserverd.<br>
 * @version 2.0.0
 */

var BrowserExt = 
{
  browserVersion: 3
};
BrowserExt.OnloadFunc = null;
BrowserExt.CaptreFunc = null;

/**
 * ライブラリの初期化 
 * @param なし
 * @returns なし<br>
 *■補足<br>
 * 　　【 呼び出しタイミングと条件】<br>
 * 　　・本関数は、ユーザーのonloadイベント処理内で実行されること。<br>
 * 　　・本関数を実行して初期化しても、onloadコンテキスト内では、他関数は使用できない。<br>
 * 　　・他関数を使用する場合にはwindow.setTimeout()を利用し100msec以降に呼び出すこと。<br>
 */
BrowserExt.Init = function()
{
    var pos = navigator.userAgent.indexOf("FX-EWB/",0);
    if( pos >= 0 ){
      this.browserVersion = navigator.userAgent.substr(pos+7,1);
    }
    if( document.EwbClass == null ) {
         var object = document.createElement("object");
         object.setAttribute("classid","EwbClass");
         object.setAttribute("name","EwbClass");
         object.setAttribute("width","0");
         object.setAttribute("height","0");
         document.body.appendChild(object);
    }
    if( this.browserVersion == 3 ) {
         var link = document.createElement("a");
         link.setAttribute("href","mailto:");
         link.setAttribute("id","EwbA");
         document.body.insertBefore(link, document.body.firstChild);
    }
}
/**
 * ブザーを鳴らす 
 * @param type ブザー音の種類（0：Valid(許可）音、1：Invalid(禁止）音）を示す数値<br>
 * @returns なし
 * @addon
 */
BrowserExt.Beep = function(type)
{
    if( type == 0 ) {
        document.EwbClass.setBeep(0);
    }
    else if( type == 1 ) {
        document.EwbClass.setBeep(1);
    }
}
/**
 * 現在のAcceptLanguage情報を得る 
 * @param なし
 * @returns 取得したAcceptLanguageの文字列。取得できなかった場合はnullを返す。<br>
 *■補足<br>
 * 　　・FXテリトリーでは”ja”,”en”,”ko”,”zh-tw”,”zh-cn”の中から、以下のように設定される。<br>
 *	①,②,③<br>
 *	①＝表示言語<br>
 *	②＝初期表示言語<br>
 *	③＝en<br>
 *	なお、重複した場合は①②③の優先順位で省略される。<br>
 *	例1）①：en、②：ja、③：enの場合は”en,ja”になる。<br>
 *	例2）①：ja、②：ja、③：enの場合は“ja,en”になる。<br>
 *	※「表示言語」は、現在の操作パネル表示言語。「初期表示言語」は、製品の出荷時に設定される<br>
 *	Default言語を示す。<br>
 * @addon
 */
BrowserExt.GetAcceptLanguage = function()
{
    if( this.browserVersion == 3 ) {
        return document.EwbClass.getAcceptLang();
    }
    else {
        return navigator.language;
    }
}
/**
 * @private
 */
BrowserExt.IsExistanceCsv = function(fname)
{
    return false;
}
/**
 * @private
 */
BrowserExt.InstallCsv = function(fname,body)
{
    return false;
}
/**
 * 再度、ハードキー押下イベントを取得するためにFocusを当てる 
 * @param なし
 * @returns なし<br>
 *■補足<br>
 *      ・現在、仕様により、以下の動作を行った場合は、ハードキー押下<br>
 *	イベントが取得できなくなるため、この関数を呼び出すことで、<br>
 *	イベント取得を再度有効にする。<br>
 *	・A,TEXTAREA,INPUT,SELECTの操作を行って、Focusが移動した場合。<br>
 * @addon
 */
BrowserExt.FocusKeyEvent = function()
{
    window.setTimeout("BrowserExt.Focus()",100);
}
/**
 * ハードーキー押下イベント通知関数を登録する 
 * @param callback 通知関数は、以下の形式のJavaScript関数である。<br>
 * BOOL callback（keyCode）
 * @returns なし<br>
 *■補足<br>
 * 　　・keyCodeは押下されたハードキーの種類により以下の数値がセットされる。<br>
 * <table border=2>
 * 　　<tr><th>キー種別</th>	<th>コード(Hex)</th>	<th>キー種別</th>			<th>コード(Hex)</th></tr>
 * 　　<tr><td>０</td>			<td>0x30</td>			<td>８</td>					<td>0x38</td></tr>
 * 　　<tr><td>１</td>			<td>0x31</td>			<td>９</td>					<td>0x39</td></tr>
 * 　　<tr><td>２</td>			<td>0x32</td>			<td>＊</td>					<td>0x2a</td></tr>
 * 　　<tr><td>３</td>			<td>0x33</td>			<td>＃</td>					<td>0x23</td></tr>
 * 　　<tr><td>４</td>			<td>0x34</td>			<td>DIAL_PAUSE</td>			<td>0x2c</td></tr>
 * 　　<tr><td>５</td>			<td>0x35</td>			<td>C</td>					<td>0x08</td></tr>
 * 　　<tr><td rowspan=2>６</td>			<td rowspan=2>0x36</td>			<td rowspan=2>スタート</td>			<td>0xE002(Inspirium)</td></tr>
<tr><td>0x1002(ANT)</td></tr>
 * 　　<tr><td rowspan=2>７</td>			<td rowspan=2>0x37</td>			<td rowspan=2>ClearAll（1回目）</td>	<td>0xE000(Inspirium)</td></tr>
<tr><td>0x1000(ANT)</td></tr>
 * </table>
 *     なお、スタートキー、ClearAllキーのkeyCodeは、カスタム･サービスからのみ取得可能。<br>
 *     外部アクセスの場合は取得できない。
 * @addon
 */
BrowserExt.SetCaptureEvent = function(callback)
{
    BrowserExt.CaptreFunc = callback;

    if( this.browserVersion == 3 ) {
        var node = document.getElementById("EwbA");
        if( node.getAttributeNode("onkeydown") != null ) {
            var okd = node.getAttribute("onkeydown");
            node.setAttribute("onkeydown","BrowserExt.GetHardKey();"+okd+" return false;");
        }
        else {
            node.setAttribute("onkeydown","BrowserExt.GetHardKey()");
        }
        BrowserExt.FocusNode = node;
        BrowserExt.FocusKeyEvent();
    }
    else {
        document.onkeypress = BrowserExt.GetHardKey;
    }
}
/**
 * @private
 */
BrowserExt.Focus = function()
{
    if( this.browserVersion == 3 ) {
        if(BrowserExt.FocusNode.focus != null) {
            BrowserExt.FocusNode.focus();
        }
    }
    else {
        window.focus();
    }
}
/**
 * 機器固有の情報を取得する 
 * @param なし
 * @returns 以下の情報を返す<br>
 * bit0：カスタムサービススクリプトがDownloadインストール可能の場合は１、<br>
 * そうでない場合は０がセットされる<br>
 * @addon
 */
BrowserExt.GetDeviceInformations = function()
{
    return document.EwbClass.getDeviceInfo();
}
/**
 * 特定のサービス画面に遷移する 
 * @param service サービス画面名称。以下の名称が指定できる。<br>
 * "allservice"
 * @returns 以下の情報を返す<br>
 * サービスへの画面遷移が成功したらtrue、<br>
 * そうでない場合はfalseがセットされる<br>
 * @addon
 */
BrowserExt.SetScreenChange = function(service)
{
    var name = service;
    if( service == null || service == "" ) {
        name = "allservice";
    }
    if (navigator.userAgent.indexOf("FX-EWB/", 0) != -1) {
    	return document.EwbClass.screenChange(name);
    } 
}
/**
 * @private
 */
BrowserExt.SetLogMessage = function(msg,point)
{
    return document.EwbClass.print(msg,point);
}
/**
 * @private
 */
BrowserExt.GetHardKey = function()
{
    var kcode = event.which;
    if( this.browserVersion == 3 ) {
        kcode = event.keyCode;
    }
    //alert(kcode);
    if( BrowserExt.CaptreFunc != null ) {
        BrowserExt.CaptreFunc(kcode);
    } 
}
/**
 * @private
 */
//BrowserExt.OnloadFunc = window.onload;
BrowserExt.Main = function()
{
    BrowserExt.Init();
    if(BrowserExt.OnloadFunc != null) {
        BrowserExt.OnloadFunc();
    }
}
//window.onload = BrowserExt.Main;



