/**
 *
 * Copyright (c) 2011, Fuji Xerox Co., Ltd.  All Rights Reserved.
 *
 * Copyright protection claimed includes all forms and matters of
 * copyrightable material and information now allowed by statutory or
 * judicial law or hereafter granted, including without limitation, material
 * generated from the software programs which are displayed on the screen
 * such as icons, screen display looks, etc.
 */
// ==========================================================
// namespace declaration
// ==========================================================
var HTTP;
if(!HTTP) {HTTP = {}; }

var CONTENT_TYPE_DEFAULT	=	"text/xml; charset=utf-8";	//HTTP Content-Type

var TIMEOUT = {
	MAX				:	30000,
	MIN				:	5000
};
var sendAbort				=	null;

// ==========================================================
// public member functions
// ==========================================================
HTTP.send = function(url, options, msg) {
	HTTP.parseRequestParam(url, options, msg);

	var xhr = new XMLHttpRequest();
	xhr.callback = HTTP.localCallback;
	xhr.onreadystatechange = function() {
		if (xhr.readyState == 4) {
			if ((xhr.status == 200 || xhr.status == 500) && xhr.responseText != null) {
				xhr.callback(JSON.parse(xhr.responseText), options);
			} else {
				alert("Error: status: " + xhr.status + " text: " + xhr.responseText);
			}
		}
	};
	xhr.open("post", options.sdkTargetURL, true);
	xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
	xhr.send(options.sdkReqParam);
};

HTTP.parseRequestParam = function(targetURL, opt, param) {
	//alert("###########REQUEST###########\n"
	//		+ opt.soapAction
	//		+ "\n###########REQUEST###########\n"
	//		+ param);
	var baseURL = "http://" + location.host + "/emulator/ttnj";
	var parser = new DOMParser();
	var dom = parser.parseFromString(param, "application/xml");

	opt.sdkTargetURL = targetURL; //デフォルトは通常のURL
	opt.sdkReqParam = param; //デフォルトは通常のparam

	if (HTTP.endsWith(opt.soapAction, "statusConfig#GetAttribute")) {
		// ==========================================================
		// GetAccountConfig
		// ==========================================================
		opt.sdkTargetURL = baseURL + "/GetAccountConfig";
		opt.sdkReqParam = "";
	} else if (HTTP.endsWith(opt.soapAction, "user#GetUser")) {
		// ==========================================================
		// GetUser
		// ==========================================================
		opt.sdkTargetURL = baseURL + "/GetUser";
		opt.sdkReqParam = "";
	} else if (HTTP.endsWith(opt.soapAction, "information#GetUserInformation")) {
		// ==========================================================
		// GetUserInformation
		// ==========================================================
		var userId = "";
		if (dom.getElementsByTagName("UserID") != null) {
			var e = dom.getElementsByTagName("userid");
			userId = e[0].firstChild.nodeValue;
		}
		opt.sdkTargetURL = baseURL + "/GetUserInformation";
		opt.sdkReqParam = "userId=" + userId;
	} else if (HTTP.endsWith(opt.soapAction, "#GetAccountingRelation")) {
		// ==========================================================
		// GetAccountingRelation
		// ==========================================================
		var userId = "";
		if (dom.getElementsByTagName("userid") != null) {
			var e = dom.getElementsByTagName("userid");
			if (e[0].firstChild != null && e[0].firstChild.nodeValue != null) {
				userId = e[0].firstChild.nodeValue;
			}
		}
		opt.sdkTargetURL = baseURL + "/GetAccountingRelation";
		opt.sdkReqParam = "userId=" + userId;
	}
};

HTTP.localCallback = function(res, opt) {
	var xml = null;
	var text = null;

	if (res.ok != true) {
		var resp = {
			responseXML : xml,
			responseText : text,
		};
		opt.successHandler(resp);
		return;
	}

	if (HTTP.endsWith(opt.soapAction, "statusConfig#GetAttribute")) {
		// ==========================================================
		// GetAttribute:AccountConfiguration
		// ==========================================================
		xml = document.createElement("GetAttributeResponse");
		HTTP.createElement(xml, "Attribute", "Custom");
	} else if (HTTP.endsWith(opt.soapAction, "user#GetUser")) {
		// ==========================================================
		// GetUser
		// ==========================================================
		xml = document.createElement("GetUserResponse");
		var user = document.createElement("User");
		xml.appendChild(user);
		HTTP.createElement(user, "UserID", res.userId);
		HTTP.createElement(user, "DisplayName", res.userName);
		HTTP.createElement(user, "CardID", res.cardId);
		HTTP.createElement(user, "Index", res.index);
		HTTP.createElement(user, "Password", res.networkPassword);
		HTTP.createElement(user, "EncryptedPassword", res.encryptedPassword);
		HTTP.createElement(user, "Roles", res.roles);
		HTTP.createElement(user, "ICCardID", res.icCardId);
		HTTP.createElement(user, "SubUserIDs", res.subUserId);
		HTTP.createElement(user, "RelatedUserID", res.relatedUserID);
	} else if (HTTP.endsWith(opt.soapAction, "information#GetUserInformation")) {
		// ==========================================================
		// GetUserInformation
		// ==========================================================
		xml = document.createElement("GetUserInformationResponse");
		var ui = document.createElement("UserInformation");
		xml.appendChild(ui);
		HTTP.createElement(ui, "UserID", res.userId);
		HTTP.createElement(ui, "SpecialUserID", res.specialUserID);
		HTTP.createElement(ui, "AuthorizedRightForInternal", res.authorizedRightForInternal);
		HTTP.createElement(ui, "AuthorizedRightForExternal", res.authorizedRightForExternal);
		HTTP.createElement(ui, "AuthorizedRight", res.authorizedRight);
		HTTP.createElement(ui, "AccountRecord", res.accountRecord);
		HTTP.createElement(ui, "UserNameForPoolService", res.userNameForPoolService);
		HTTP.createElement(ui, "MailAddress", res.emailAddress);
		HTTP.createElement(ui, "PasswordForPoolService", res.networkPassword);
		HTTP.createElement(ui, "EncryptedPasswordForPoolService", res.encryptedPassword);
		HTTP.createElement(ui, "Roles", res.roles);
		HTTP.createElement(ui, "HomeDirectory", res.homeDirectory);
		HTTP.createElement(ui, "RelatesAccountID", res.relatesAccountID);
		HTTP.createElement(ui, "Permissions", res.permissions);
		HTTP.createElement(ui, "RemoteAuthentication", res.remoteAuthentication);
		HTTP.createElement(ui, "SubUserIDs", res.subUserId);
		HTTP.createElement(ui, "DisplayName", res.userName);
		if (res.accounting != null && res.accounting.length > 0) {
			var accounting = document.createElement("Accounting");
			ui.appendChild(accounting);
			for (var i = 0; i < res.accounting.length; i++) {
				HTTP.createElement(accounting, "UserID", res.accounting[i].userId);
				HTTP.createElement(accounting, "AccountID", res.accounting[i].accountID);
			}
		}
	} else if (HTTP.endsWith(opt.soapAction, "#GetAccountingRelation")) {
		// ==========================================================
		// GetAccountingRelation
		// ==========================================================
		xml = document.createElement("GetAccountingRelationResponse");
		var ar = document.createElement("AccountingRelation");
		xml.appendChild(ar);
		if (res.accountingRelation != null) {
			var accounting = document.createElement("Accounting");
			ar.appendChild(accounting);
			for (var i = 0; i < res.accountingRelation.length; i++) {
				HTTP.createElement(accounting, "UserID", res.accountingRelation[i].userId);
				HTTP.createElement(accounting, "AccountID", res.accountingRelation[i].accountId);
			}
		}
	}

	if (xml != null) {
		text = XMLToString(xml);
	}
	//alert("###########RESPONSE###########\n" + text);
	var resp = {
		responseXML : xml,
		responseText : text,
	};
	opt.successHandler(resp);
};

HTTP.createElement = function(root, key, value) {
	var elm = document.createElement(key);
	if (value != null) {
		elm.appendChild(document.createTextNode(value));
	}
	root.appendChild(elm);
}

HTTP.endsWith = function(str, suffix) {
	var i = str.length - suffix.length - 1;
	var j = str.indexOf(suffix);
	return (i >= 0 && j != -1 && j == i);
}

HTTP.Options = function() {
	var opt = {
		successHandler : null,
		errorHandler : null,
		timeoutHandler : null,
		async : true,
		method : "post",
		soapAction : null,
		contentType : CONTENT_TYPE_DEFAULT,
		localAccess : true,
		user : null,
		password : null,
		timeout : 10000,
		sdkTargetURL : null,	// XCP-SDK only
		sdkReqParam : null,		// XCP-SDK only
	};
	return opt;
};

