﻿/** 
 * @fileoverview デバイス内部で発生したイベント受信・通知のためのクラス定義<br>
 * デバイス内部で発生したイベントをDOMイベントとして受信し、<br>
 * 通知するためのJavaScriptライブラリ<br>
 * 
 * @author Copyright(C) 2011 Fuji Xerox Co., Ltd. All rights reserved.
 * @version 1.0.0
 * @lang ja
 */

 /** 
 * デバイス内部で発生したイベント受信/通知のための名前空間定義
 * @namespace DOMイベント受信/通知のための名前空間定義
 * @static
 * @lang ja
 */

var SystemEventLib = {
	/**
	 *	SystemEventLibのバージョンを表す
	 *	@constant
	 *	@type String
	 *  @lang ja
	 */
  	version : "1.0.0",

	
/**
 * 各イベント種別。プロパティはlistenerのみ
 * @private
 */
  AuthEvent: {listener : null},
	
  CstmAuthAccEvent : {listener : null},
	
  ChgScreenResultEvent : {listener : null},

  CstmSvcDispEvent : {listener : null }

};

/**
 * ライブラリからのイベントを受け取るための通知登録を行う。<br>
 * 各イベント種別に対し、登録可能なイベントハンドラの数は１つだけである。<br>
 * 同一イベント種別の登録指示の場合は、後から指示された
 * イベントハンドラにより上書きされる。<br>
 *
 * @param {String} eventname イベント種別<br>
 * AuthEvent:認証/集計状態に関するイベント(カスタム認証以外)<br>
 * CstmAuthAccEvent:カスタム認証時のアクセサリ有効・無効の通知、カスタム認証時の認証状態の通知<br>
 * ChgScreenResultEvent:画面切り替え指示の結果通知<br>
 * CstmSvcDispEvent:表示画面（カスタムサービス画面/デバイスが提供する画面）の検知結果通知<br>
 * <font color="#FF0000">(V1.0.0では上記イベント種別のうちCstmAuthAccEventのAccountStateプロパティのみをサポート)</font><br>
 * 
 * @param {Function} callback
 * 登録されたイベントの更新が発生したときに呼び出されるコールバック関数<br>
 * 本コールバック関数は、以下の形のJavaScript関数である必要がある。<br>
 * void	callback ( <i>eventname</i> , <i>obj</i> ) <br>
 * eventname : イベント種別<br>
 * obj : イベント発生時に返されるオブジェクト。プロパティ詳細と取りうる値は以下。<br><br>
 * 
 * obj(AuthEvent)のプロパティ
 * <blockquote><b>State</b> デバイス内部の認証状態を示す<br>
 * Enbl : 認証済み<br>
 * Dsbl : 未認証（デバイス内部でのキャンセル完了含む)<br>
 * PwdReq : パスワード待ち<br>
 * AuthNg : 認証失敗<br>
 * Proc : 照合中<br>
 * CnclProc : キャンセル中<br>
 * </blockquote>
 * <blockquote><b>FailCh</b> 認証失敗時のChain番号</blockquote>
 * <blockquote><b>FailLk</b> 認証失敗時Link番号</blockquote>
 * <blockquote><b>UsrType</b> ユーザ種別<br>
 *  CO : 一般ユーザ<br> 
 *  KO : 機械管理者<br>
 * </blockquote>
 * <blockquote><b>UsrId</b> ユーザID<br></blockquote>
 * <blockquote><b>IccdId</b> ICカードID<br></blockquote>
 * <blockquote><b>NgRsn</b> 認証失敗の理由<br>
 * InputWrong：入力不正(認証失敗)<br>
 * AccountIdWrong：AccountId不正<br>
 * ServerNotAvailable：サーバーアクセス失敗<br>
 * AccountLostInfo：ユーザーアカウントが登録されていない<br>
 * InvaliedSetup：設定不良<br>
 * NoAuth：使用権限無し<br>
 * Other：論理的にありえるが、発生確率が極めて低い場合のエラー<br>
 * InvalidConnectionInfo：通信情報不整合<br>
 * ForceLogout：強制ログアウトがCAサーバーから要求された<br>
 * NoSpace：ユーザーを作る領域がない<br>
 * CableProblem：ケーブル抜け<br>
 * NotInitNetStack:プロトコルスタックが未初期化<br>
 * NotActiveComm:能動的通信が不可能<br>
 * </blockquote>
 * <blockquote><b>ExtSvrFailCh</b> 外部認証失敗時のChain番号<br></blockquote>
 * <blockquote><b>ExtSvrFailLk</b> 外部認証失敗時のLink番号<br></blockquote>
 * <blockquote><b>PrevState</b> デバイス内部の前回の認証状態を示す(とりうる値はStateプロパティを参照のこと)</blockquote>
 *  
 * obj(CstmAuthAccEvent)のプロパティ<br>
 * #AccountState/AuthStateいずれかのプロパティのみ存在する<br>
 *
 * <blockquote><b>AccountState</b>アクセサリ有効・無効通知<br>
 * Enbl : アクセサリ有効<br>
 * Dsbl : アクセサリ無効<br>
 * </blockquote>
 * <blockquote><b>AuthState</b> 認証状態<br>
 * None : 未認証<br>
 * Enbl : 認証済み<br>
 * Proc : 認証照合中<br>
 * CnclProc : 照合キャンセル中
 * </blockquote>
 * 
 * obj(ChgScreenResultEvent)のプロパティ
 * <blockquote><b>Cmd</b> リクエストされたコマンド文字列<br>
 * popup : ポップアップ要求<br></blockquote>
 * <blockquote><b>Param</b> コマンドに対するパラメータ<br>
 * lang : 表示言語切り替え画面<br>
 * tools_menu : 仕様設定画面</blockquote>
 * <blockquote><b>Rslt</b> 実行結果<br>
 * OK : 成功<br>
 * NG : 失敗<br></blockquote>
 * <blockquote><b>NgRsn</b> 失敗時の理由(RsltがNGの場合のみ存在する）<br>
 * FatalErr : 致命的内部エラー発生<br>
 * NoPermission : 実行ユーザに操作権限がない<br>
 * TmpRefuse : デバイスが要求を受け付けられる状態にない<br>
 * </blockquote>
 * 
 * obj(CstmSvcDispEvent)のプロパティ
 * <blockquote>
 * true : カスタムサービスの画面表示<br>
 * false : デバイスが提供する画面表示<br>
 * </blockquote>
 * 
 * @returns {Bool} true:登録成功<br>
 * false:登録失敗(eventnameが不正)
 *
 * @addon
 * @static
 * @lang ja
 */
SystemEventLib.AddNotify = function (eventname, callback) {

	if ( !SystemEventLib[eventname] ) {
		return false;
	}

	SystemEventLib.setEventListener(eventname, callback);
	return true;
};

/**
 * ライブラリからのイベント通知登録の解除を行う。<br>
 *
 * @param {String} eventname イベント種別<br>
 * AuthEvent:カスタム認証以外の認証/集計状態に関するイベント<br>
 * CstmAuthAccEvent:カスタム認証時のアクセサリ有効・無効の通知、カスタム認証時の認証状態の通知<br>
 * ChgScreenResultEvent:画面切り替え指示の結果通知<br>
 * CstmSvcDispEvent:表示画面（コンテンツの画面/デバイスが提供する画面）の検知結果通知
 * 
 * @returns {Bool} true:登録解除成功<br>
 * false:登録解除に失敗(eventnameが不正)
 *
 * @addon
 * @static
 * @lang ja
 */
SystemEventLib.RemoveNotify = function(eventname){

	if ( !SystemEventLib[eventname] ) {
		return false;
	}
	SystemEventLib.removeEventListener(eventname);
	return true;
};

/**
 * eventnameに応じてイベントターゲットにリスナーを登録する
 * @private
 */
SystemEventLib.setEventListener = function(eventname, callback) {
	if ( eventname == "CstmSvcDispEvent" ) {
		document.addEventListener("suspend", SystemEventLib.onDisplayEvent, false);
		document.addEventListener("resume", SystemEventLib.onDisplayEvent, false);
	}
	else {
		document.addEventListener("FxSystemEvent", SystemEventLib.onSystemEvent, false);
	}
	SystemEventLib[eventname].listener = callback;
	return;
};

/**
 * eventnameに応じてイベントターゲットのリスナー登録を解除する
 * @private
 */
SystemEventLib.removeEventListener = function(eventname) {
	if ( eventname == "CstmSvcDispEvent" ) {
		document.removeEventListener("suspend", SystemEventLib.onDisplayEvent, false);
		document.removeEventListener("resume", SystemEventLib.onDisplayEvent, false);
	}
	else {
		document.removeEventListener("FxSystemEvent", SystemEventLib.onSystemEvent, false);
	}
	SystemEventLib[eventname].listener = null;
	return;
};


/**
 * 登録済みのイベントハンドラを実行する
 * @private
 */
SystemEventLib.onEvent = function (eventname, obj) {

	if ( SystemEventLib[eventname].listener && obj != null) {
		SystemEventLib[eventname].listener(eventname, obj);
	}
	return;
};

/**
 * @private
 */
SystemEventLib.onSystemEvent = function (event){
/*
alert("onSystemEvent()");
alert(event);
alert(JSON.stringify(event, null, " "));
*/
	var _attrName;
	_attrName = event.detail.attrName;

	if ( !_attrName ) {
alert("return !_attrName");
		return;
	}

	var _attrVal;
	_attrVal = event.detail.attrValue;
	if ( !_attrVal ) {
alert("return !_attrVal");
		return;
	}
	_attrVal = eval('('+_attrVal+')');
	var _obj = new Object();

	switch ( _attrName ){
	  case "AAAAccountState" :
		var _category = _attrVal["Ctgry"];

		if ( _category == "All" ) {
			_obj = getAuthObj(_attrVal);
			SystemEventLib.onEvent("AuthEvent", _obj);
		}
		if ( _category == "Acc" ) {
			_obj = getCstmAuthAccObj(_attrVal);
			SystemEventLib.onEvent("CstmAuthAccEvent", _obj);
		}
		break;

	  case "AAAAuthenticationState" :
		var _client = _attrVal["Client"];

		if ( _client == "Xcp" ) {
			_obj = getCstmAuthXcpObj(_attrVal);
			SystemEventLib.onEvent("CstmAuthAccEvent", _obj);
		}
		break;
		
	  case "ChangeScreenResult" :
		_obj = getChgScreenResultObj(_attrVal);
		SystemEventLib.onEvent("ChgScreenResultEvent", _obj);
		break;
		
	  default:
		/* そのほかのイベントが来ても何もしない */
		break;
	}

};

/**
 * @private
 * @param {Object} event イベントオブジェクト
 * @param {Bool} obj 通知結果
 */
SystemEventLib.onDisplayEvent = function(event){
	
	var _obj = null;

	if ( event.type == "resume" ) {
		_obj = true;
	}
	if ( event.type == "suspend" ) {
		_obj = false;
	}
	SystemEventLib.onEvent("CstmSvcDispEvent", _obj);
	return;
};

/**
 * AuthEvent用にフィルタリングしたオブジェクトを返す
 * @private
 */
function getAuthObj(attrValue) {
/*
alert("getAuthObj()");
alert(attrValue);
alert(JSON.stringify(attrValue, null, " "));
*/
	var _state = attrValue["State"];
	if ( !_state ) { return null; }
	if ( _state == "Lost") {
		return null;
	}
	
	/* Dsblが2回連続した場合は、2回目の通知は行われない*/
	var _prevState = attrValue["PrevState"];
	if ( !_prevState ) { return null; }
	if ( _state == "Dsbl" && _prevState == "Dsbl") {
		return null;
	}
	
	var _usrType = attrValue["UsrType"];
	if ( !_usrType ) { return null; }
	if ( _usrType != "CO" && _usrType != "KO"){
		return null;
	}

	var _retObj = new Object();

	for (var prop in attrValue){
		switch(prop){
		  case "State":
		  case "PrevState":
			
			/* Cnslの場合はDsblとして扱う*/
			if ( attrValue[prop] == "Cncl" ){
				_retObj[prop] = "Dsbl";
			} else {
				_retObj[prop]= attrValue[prop];
			}
			
			break;
		  case "NgRsn":
			_retObj[prop] = SystemEventLib.AccountNgRsn[attrValue[prop]];
			break;

		  case "FailCh" :
		  case "FailLk" :
		  case "UsrType":
		  case "UsrId":
		  case "IccdId":
		  case "ExtSvrFailCh":
		  case "ExtSvrFailLk":
			_retObj[prop] = attrValue[prop];
			break;
			
		  default:
			/* その他のパラメータは無視 */
			break;
		}
	}

	return _retObj;
};

/**
 * CstmAuthAccEvent用にフィルタリングしたオブジェクトを返す
 * (AccountState)
 * @private
 */
function getCstmAuthAccObj(attrValue){
/*
alert("getCstmAuthAccObj()");
alert(attrValue);
alert(JSON.stringify(attrValue, null, " "));
*/
	var _retObj = new Object();
	for ( var prop in attrValue ){
		switch ( prop ) {
		  case "State" :
			switch ( attrValue["State"] ) {
			  case "Enbl" :
			  case "Dsbl":
				_retObj["AccountState"] = attrValue[prop];
				break;
				
			  default:
				break;
			}
			break;

		  default:
			break;
		}
	}
	return _retObj;
};

/**
 * CstmAuthAccEvent用にフィルタリングしたオブジェクトを返す
 * (AuthState)
 * @private
 */

function getCstmAuthXcpObj(attrValue){
/*
alert("getCstmAuthXcpObj()");
alert(attrValue);
alert(JSON.stringify(attrValue, null, " "));
*/
	var _retObj = new Object();
	for ( var prop in attrValue ){
		switch ( prop ) {
		  case "State" :
			switch( attrValue["State"] ) {
			  case "None":
			  case "Ok":
			  case "Processing":
			  case "CancelProc":
				_retObj["AuthState"] = SystemEventLib.AuthStateVal[attrValue[prop]];
				break;

			  default:
				break;
			}
		}
	}
	return _retObj;
};

/**
 * ChgScreenResultEvent用にフィルタリングしたオブジェクトを返す
 * @private
 */
function getChgScreenResultObj(attrValue){
	
	var _retObj = new Object();
	for (var prop in attrValue){
		switch( prop ){
		  case "Cmd" :
			switch( attrValue["Cmd"] ) {
			  case "popup" : 
				_retObj[prop] = attrValue[prop];
				break;

			  default:
				break;
			}
			break;
			
		  case "Param":
			switch( attrValue["Param"] ) {
			  case "lang":
			  case "tools_menu":
				_retObj[prop] = attrValue[prop];
				break;

			  default:
				/* Unknownなど */
				break;
			}
			break;
			
		  case "Rslt":
			_retObj[prop] = attrValue[prop];
			break;
			
		  case "NgRsn":
			_retObj[prop] = SystemEventLib.PopupNgRsn[attrValue[prop]];
			break;

		  default:
			break;
		}
	}
	return _retObj;
};

/**
 * AccountStateEventのNgRsn読み替え
 * @private
 */
SystemEventLib.AccountStateNgRsn = {
	InpWrng : "InputWrong",
	AccIdWrng : "AccountIdWrong",
	SvrErr: "ServerNotAvailable",
	AccLost: "AccountLostInfo",
	Invalid:"InvalidSetup",
	NoAuth:"NoAuth",
	Other:"Other",
	InvConInf:"InvalidConnectionInfo",
	FrcLogout:"ForceLogout",
	NoSpace:"NoSpace",
	CablePrblm:"CableProblem",
	NetNoInit:"NotInitNetStack",
	CommInactive:"NotActiveComm"
};

/**
 * ChgScreenResultEventのNgRsn読み替え
 * @private
 */
SystemEventLib.PopupNgRsn = {
	NgInternal:"FatalErr",
	NgOnTools:"FatalErr",
	NgChangModeFail:"TmpRefuse",
	NgNoPermission:"NoPermission"
};


/**
 * CstmAuthAccEvent ( AAAAuthenticationStateEvent ) のState読み替え
 * @private
 */
SystemEventLib.AuthStateVal = {
	None : "None",
	Ok : "Enbl",
	Processing : "Proc",
	CancelProc : "CnclProc"
};