/* apeos.js
 * Copyright (C) Fuji Xerox Co., Ltd. 2009. All rights reserved.
 *
 * Java Script Library for Apeos.
 *
 * HISTORY:
 * Version1.0 Mar 31, 2009 Created.
 * Version1.1 Jul 17, 2012 Modifed.
 */

var apeos = 
{
  type: 0,
  lcd:   [ "Svga","Wvga" ],
  width: [ 800,800 ],
  height:[ 560,440 ],
  push: false
};

/**
 * Get LCD size type
 *
 * GetLCDTYpe()
 *
 * return 0:SVGA,1:WVGA,-1:UNKOWN
 */

apeos.getLcdType = function()
{
  var ver = this.getBrowserVersion();
  if( ver == 5 ) {
  if( document.width == 800 ){
    if( document.height >= 560 ){
      return 0;
    }
    else{
      return 1;
    }
  }
  else{
    return -1;
  }
  }
  else {
  if( screen.width == 800 ){
    if( screen.height >= 560 ){
      return 0;
    }
    else{
      return 1;
    }
  }
  else{
    return -1;
  }
  }
}

/**
 * beep sound for selection.
 *
 * beep( type )
 * type: 0=valid,1=Invalid
 */

apeos.beep = function(type)
{
  var ver = this.getBrowserVersion();
  if( ver == 3 || ver == 4 ) {
  if( document.EwbClass != null ) {
     if( type == 0 ) {
       document.EwbClass.setBeep(0);
     }
     else {
       document.EwbClass.setBeep(1);
     }
  }
  else {
    if( type == 0 ) {
        EwbClass.setBeep(0);
    }
    else if( type == 1 ) {
        EwbClass.setBeep(1);
    }
  }
  }
}

/**
 * get current language.
 *
 * getAcceptLanguage( )
 * 
 * return ISO-639 code.
 */

apeos.getAcceptLanguage = function()
{
  var ver = this.getBrowserVersion();
  if( ver == 5 ) {
        if(navigator.language == "zh-CN") {
            return("zh-cn");
        }
        else if(navigator.language == "zh-TW") {
            return("zh-tw");
        }
        else {
            return navigator.language.substring(0,2);
        }
  }
  else {
    return navigator.language;
  }
}

/**
 * transfer to other screen.
 *
 * setScreenChange(service)
 * 
 * service: "allservice" or null
 */

apeos.setScreenChange = function(service)
{
  var name = service;
  if( service == null || service == "" ) {
    name = "allservice";
  }
  var ver = this.getBrowserVersion();
  if( ver == 5 ) {
    return EwbClass.screenChange(name);
  }
  if( document.EwbClass != null ) {
    return document.EwbClass.screenChange(name);
  }
  return false;
}

/**
 * get browser major version.
 *
 * getBrowserVersion( )
 *
 * return: browser major version.
 */

apeos.getBrowserVersion = function()
{
  var ver = -1; /* default version */
  var pos = navigator.userAgent.indexOf("FX-EWB/",0);
  if( pos >= 0 ){
    ver = navigator.userAgent.substr(pos+7,1);
  }
  return ver;
  /**return 4;**/
}

/**
 * setup CSS&JavaScript file based on browser version.
 *
 * setupLibrary( path )
 *
 * path: directory which is existting css&js file.
 */

apeos.setupLibrary = function( path )
{
  var ver = this.getBrowserVersion();
  var fname = path + "v" + ver + "widget.css";
  document.write("<link rel='stylesheet' type='text/css' href='",fname,"'>");
  fname = path + "v" + ver + "widget.js";
  document.write("<script type='text/JavaScript' src='",fname,"'></script>");
}

/**
 * setup CSS file based on display size.
 *
 * setupStyleSheet( name, path )
 *
 * name: CSS file name (actual name is "Svga" or "Wvga" +name+".css")
 * path: directory which is existting css file.
 */

apeos.setupStyleSheet = function(name,path)
{
  var ver = this.getBrowserVersion();
  apeos.type = apeos.getLcdType();
  if( apeos.type == -1 ) {
    apeos.type = 1;
  } 
  var fname = path + "v" + ver + apeos.lcd[apeos.type] + name + ".css";
  document.write("<link rel='stylesheet' type='text/css' href='",fname,"'>");
}

/**
 * CSS change when OnMouseDown/OnMouseUp event is occured.
 *
 * ButtonDown( evt ),ButtonUp( evt )
 *
 * evt: event object.
 */

apeos.ButtonDown = function(event)
{
  apeos.push = true;
  if( event.currentTarget.className == "OFF" ) {
    event.currentTarget.className = "ON";
  }
  apeos.beep(0);
}
apeos.ButtonUp = function(event)
{
  if( event.currentTarget.className == "ON" ) {
    event.currentTarget.className = "OFF";
  }
  apeos.push = false;
}
apeos.ButtonOut = function(event)
{
  if( apeos.push ) {
    apeos.ButtonUp(event);
  }
}

apeos.setVisibleForPrintHistory = function(visible)
{
  if( this.getBrowserVersion() == 5 ) {
    return EwbClass.setVisibleForPrintHistory(visible);
  }
};

apeos.getAllocatedMemorySize = function()
{
    if( this.getBrowserVersion() == 4 ) {
      return ( debug.flexAllocated + debug.heapAllocated );
    }
    if( this.getBrowserVersion() == 5 ) {
      return EwbClass.getAllocatedMemorySize();
    }
};

apeos.getFreeMemorySize = function()
{
    if( this.getBrowserVersion() == 4 ) {
      return ( debug.flexFree + debug.heapFree );
    }
    if( this.getBrowserVersion() == 5 ) {
      return EwbClass.getFreeMemorySize();
    }
};

apeos.setAutoClearEnable = function()
{
    if( this.getBrowserVersion() == 5 ) {
      EwbClass.setAutoClearEnable ();
    }
};

apeos.getDisplayWidth = function()
{
    if( this.getBrowserVersion() == 5 ) {
      if(navigator.platform.indexOf("Win32",0) >= 0 ) {
        return window.outerWidth;
      }
      return screen.width;
    }
    else {
      return screen.width;
    }
};

apeos.getDisplayHeight = function()
{
  if( this.getBrowserVersion() == 5 ) {
      if(navigator.platform.indexOf("Win32",0) >= 0 ) {
        if(window.outerHeight == 440 || window.outerHeight == 480) {
          return 480;
        }
        return 600;
      }
      return screen.height;
    }
    else {
      return screen.height;
  }
};







