/* widget.js
 * Copyright (C) Fuji Xerox Co., Ltd. 2009. All rights reserved.
 *
 * Special widget library for Apeos IntrgrationPlus.
 *
 * HISTORY:
 * Mar 31, 2009  Version 1.0
 */

/******************************************************************
 * apeos object
 ******************************************************************/

/**
 * widget id number definitions
  0 : xrx-radio
  1 : xrx-checkbox
  2 : xrx-command
  3 : xrx-text
  4 : xrx-password
  5 : xrx-select

 * widget state definitions
  0 : selectable
  1 : pressed
  2 : selected
  3 : unselectable
  4 : unselectable(selected) or edit

**/

/******************************************************************
 * apeos utility object
 ******************************************************************/

apeos.constvalue = {
//  imagePath: "png/"
	imagePath: "../Lib/images/"
};

apeos.util = {
  dwdebug: false,
  fname: ["createRadio()","createCheckbox()","createCommand()","createInputText()","createInputPassword()"],
  wsize: [ 00,36,40,44,48,52,56,60,64 ], /* radio and checkbox size */
  csize: [ 00,32,40,48,56,64 ],	         /* command size */
  cfont: [ 00,20,20,30,30,30 ],          /* font size for text&password&command */
  cmarg: [ 00,6,10,9,13,17 ],            /* margin top for text&password&command */
  fsize: [ 00,20,20,20,30,30,30,30,30 ], /* font size for radio&checkbox */
  padds: [ 00,4,5,5,6,6,7,7,8 ],         /* padding size for radio&checkbox */
  margs: [ 00,0,2,4,1,2,3,5,5],          /* margin top size for radio&checkbox */
  log: "",
  msg: "",
  onload: false,
  rimage:[["BZ_RC99_1.png", "BZ_RC99_2.png", "BZ_RC99_3.png", "BZ_RC99_4.png", "BZ_RC99_5.png"],
          ["BZ_CS99_1.png", "BZ_CS99_2.png", "BZ_CS99_3.png", "BZ_CS99_4.png", "BZ_CS99_5.png"],],
  cimage:[["BX_CXX_99_11.png","BX_CXX_99_41.png","BX_CXX_99_21.png","BX_CXX_99_31.png","",
           "BX_CXX_99_32.png","BX_CXX_99_12.png","BX_CXX_99_42.png","BX_CXX_99_22.png"],
          ["BX_BXX_99_11.png","BX_BXX_99_41.png","BX_BXX_99_21.png","BX_BXX_99_31.png","",
           "BX_BXX_99_32.png","BX_BXX_99_12.png","BX_BXX_99_42.png","BX_BXX_99_22.png"]],
  fcolor: [ "#000000","#000000","#000000","#ADAAAD","#ADAAAD" ],
  bcolor: [ "#EFEFEF","#5A96EF","#83B0FA","#E6E7E6","#FFFFFF" ],
  icolor: [ "#F7F7F7","#5A96EF","#FFFFFF","#E6E7E6","#FFFFFF" ],
  curtxt: null,
  curwid: 0,
  numtxt: 0,
  widtxt: [ 0 ],
  ndetxt: [ "" ],
  pressing: false,
  lognode: null
};

apeos.util.beep = function(type)
{
    if( type == 0 ) {
        EwbClass.setBeep(0);
    }
    else {
        EwbClass.setBeep(1);
    }
}

apeos.util.addBlurListner = function(id,table)
{
    var registerd = false;

    for( var i=0; i < this.numtxt; i++ ) {
        if( this.ndetxt[i] == table ) {
            registerd = true;
            break;
        }
    }
    if( registerd == false ) {
        this.ndetxt[this.numtxt] = table;
        this.widtxt[this.numtxt] = id;
        this.numtxt++;
    }
    this.curtxt = table;
    this.curwid = id;
}

apeos.util.checkBlur = function(target)
{
    if( this.parentTable(target) != this.curtxt ) {
        this.curtxt = null;
    }
    for( var i=0; i < this.numtxt; i++ ) {
        if( this.ndetxt[i] != this.curtxt ) {
            this.textBlur(this.widtxt[i],this.ndetxt[i])
        }
    }
}

apeos.util.parentTable = function(node)
{
    for( var par = node.parentNode; par != null; par = par.parentNode ) {
        if( par.nodeName == "TABLE" ) {
            return par;
        }
    }
    return null;
}

apeos.util.copyEvent = function(evt,wid,down,up)
{
    var prm = "";
    if(evt != null) {
        if(evt.ondown != null) {
            prm += " ondown=" + evt.ondown;
        }
        if(evt.onup != null) {
            prm += " onup=" + evt.onup;
        }
        if(evt.onclick != null) {
            prm += " onck=" + evt.onclick;
        }
	if(wid == 0 || wid == 1 ) {
        if(evt.onchange != null) {
            prm += " onchg=" + evt.onchange;
        }
	}
	if(wid == 3 || wid == 4 ) {
        if(evt.onkpress != null) {
            prm += " onkpress=" + evt.onkpress;
        }
	}
    }
    prm += " onmousedown=" + down+"("+wid+")";
    prm += " onmouseup=" + up+"("+wid+")";
    prm += " onmouseout=apeos.util.mouseOut("+wid+")";
    return prm;
}

apeos.util.mouseOut = function(wid)
{
    if( this.pressing ) {
        var tab = event.currentTarget;
        var fnc = tab.getAttribute("onmouseup");
        eval(fnc);
    }
}

apeos.util.write = function(value)
{
    if(apeos.util.dwdebug == true) {
        this.print(false,value);
    }
    else {
        document.write(value);
    }
}

apeos.util.print = function(update,txt)
{
    if( apeos.util.onload == false ) {
        if( update ) {
            apeos.util.log = txt;
        }
        else {
            apeos.util.log += txt;
        }
    }
    else if( apeos.util.lognode != null ) {
        if( update ) {
	    apeos.util.lognode.innerText = txt;
        }
        else {
            apeos.util.lognode.innerText += txt;
        }
    }
}

apeos.util.showLog = function()
{
    var div = document.getElementById("debugcomment");
    if(div == null) {
        div = document.createElement("div");
        div.setAttribute("id","debugcomment");
        var txt = document.createTextNode(apeos.util.log);
        div.style.fontSize = "16px";
        div.style.color = "blue";
        div.appendChild(txt);
        document.body.appendChild(div);
        apeos.util.lognode = div;
    }
}

apeos.util.printDomTree = function(currentElement, depth)
{
    if (currentElement)
    {
        var j;
        var tagName=currentElement.tagName;
        if (tagName) {
            elog += "("+depth+")<"+tagName+"> ";
        }
        else {
            elog += "("+depth+")["+"undef"+"] ";
        }
        var i=0;
        var currentElementChild=currentElement.childNodes[i];
        while (currentElementChild)
        {
            traverseDOMTree(currentElementChild, depth+1);
            i++;
            currentElementChild=currentElement.childNodes[i];
        }
    }
}

apeos.util.setDelayedMessage = function(msg)
{
    if( apeos.util.msg == "" ) {
        apeos.util.msg = msg;
        setTimeout("apeos.util.showDelayedMessage()",1000);
    }
    else {
        apeos.util.msg += "\n";
        apeos.util.msg += msg;
    }
}

apeos.util.showDelayedMessage = function()
{
    alert(apeos.util.msg);
}

apeos.util.checkArgument = function(id,name,val)
{
    if( name == "" ) {
        this.setErrMsg(id,'Invalid [name] argument');
        return true;
    }
    if( val == "" ) {
        this.setErrMsg(id,'Invalid [val] argument');
        return true;
    }
    return false;
}

apeos.util.setErrMsg = function(id,msg)
{
    this.setDelayedMessage( msg+' in ['+this.fname[id]+'] function!');
}

apeos.util.hookResetFunc = function(id,fid,name,fname)
{
    var frm = document.getElementById(fid);
    if( frm != null ) {
      var ifn = null;
      var rfn = frm.getAttribute("onreset");
      if( rfn != "apeos.util.execResetFunc()" ) {
        frm.onreset = apeos.util.execResetFunc;
        ifn = rfn;
      }
      var fnc = fname+'("'+id+'","'+name+'")';
      var rst = frm.getAttribute("onrst");
      if( rst != "" && rst != null ) {
          if( rst.indexOf( fnc ) == -1 ) {
              rst += fnc+";";
              frm.setAttribute("onrst",rst);
          }
      }
      else {
          if( ifn != null ) {
            rst = ifn + ";" +fnc+";";
          }
          else {
            rst = fnc+";";
          }
          frm.setAttribute("onrst",rst);
      }
    }
}

apeos.util.execResetFunc = function(event)
{
  var frm = event.currentTarget;
  var rst = frm.getAttribute("onrst");
  if( rst != null ) {
//alert(rst);
    eval(rst);
  }
}

/******************************************************************
 * BUTTON methods
 ******************************************************************/

apeos.util.getImage = function(wid,ptn,size)
{
    var img;
    if( wid == 0 || wid == 1 ) {
        var imgid = parseInt(size)+1;
        img = apeos.constvalue.imagePath + this.rimage[wid][ptn];
        var ary = img.split("99");
        img = ary.join(imgid);
        return img;
    }
    else {
        var sid = ptn+1;
        var idn = 0;
        if( wid == 3 || wid == 4 ) {
            if( ptn == 4 ) {
                sid = "E";
            }
            idn = 1;
        }
        img = apeos.constvalue.imagePath + this.cimage[idn][size-1];
        var ary = img.split("99");
        img = ary.join(sid);
        return img;
    }
}

apeos.util.getPattern = function(select,disable)
{
    var ptn = 0;
    if( select && disable ) {
        ptn = 4;
    } else if( select ) {
        ptn = 2;
    } else if( disable ) {
        ptn = 3;
    } else {
        ptn = 0;
    }
    return ptn;
}

apeos.util.toggleGenerate  = function(wid,size,fid,label,id,name,val,check,dis,evt)
{
    var prm = "";
    var hdn = "";

    if( this.checkArgument(wid,name,val) ) {
        return true;
    }

    this.hookResetFunc(wid,fid,name,"apeos.util.toggleReset");
    
    if( id ) {
        prm += " id=" + id;
    }
    if( size < 1 ) {
        size = 1;
    }
    if( size > 8 ) {
	size = 8;
    }
    //size += 1;
    prm += " siz=" + size;

    if(!dis) {
        //prm += " name=" + name + " value=" + val;
        if(name != null) {
            prm += " name=" + name;
            if(check) {
                hdn += " name=" + name;
            }
            if(val != null) {
                prm += " value=" + val;
                if(check) {
                    hdn += " value=" + val;
                }
            }
        }
    }
    var ptn = this.getPattern(check,dis);
    prm += " cptn=" + ptn + " dptn=" + ptn;
    var col = 'style="color:' + this.fcolor[ptn] + ';font-size:'+this.fsize[size]+';padding-left:'+this.padds[size]+'px;margin-top:'+this.margs[size]+'px"';

    var tdw;

    if( apeos.getAcceptLanguage() == "ja" ) {
      tdw = 'style="width:' + (this.fsize[size]*label.length)+'px"';
    }
    else {
      tdw = 'style="width:' + (this.fsize[size]*label.length)/2+'px"';
    }

    var img = this.getImage(wid,ptn,size);
    var evp = this.copyEvent(evt,wid,"apeos.util.toggleMousedown","apeos.util.toggleMouseup");
    var ims = ' width="'+this.wsize[size]+'" height="'+this.wsize[size]+'"';
    apeos.util.write('<table class="xrx-radio" ' + prm + evp+'><tr><td class="image"><img src="' + img + '" '+ims+'></td><td '+tdw+'><div class="label" ' + col + '>' + label + '</div></td><td><input type="hidden" '+hdn+'></td></tr></table>');
}

apeos.util.toggleMousedown = function(id)
{
    this.pressing = true;
    var tab = event.currentTarget;
    var ptn = tab.getAttribute("cptn");
    var siz = tab.getAttribute("siz");
    var ond = tab.getAttribute("ondown");
    if(ptn == "0" || ptn == "2") {
        var img = tab.getElementsByTagName("img");
        var div = tab.getElementsByTagName("div");
        img[0].src = this.getImage(id,1,siz);
        this.execHandler(ond);
        this.beep(0);
    }
};

apeos.util.toggleMouseup = function(id)
{
    if( this.pressing ) {
    var tab = event.currentTarget;
    if( id == 0 ) {  // xrx:radio ?
        this.toggleFindGroup(id,tab.getAttribute("name"),tab);
    }
    else {
        this.toggleChange(id,tab);
    }
    this.pressing = false;
    }
};

apeos.util.toggleReset = function(id,name)
{
    var tab = document.getElementsByTagName("table");
    for(var i=0; i<tab.length; i++) {
        if(name == tab[i].getAttribute("name")) {
            this.toggleInit(id,tab[i]);
        }
    }
}

apeos.util.toggleInit = function(id,tab)
{
    var ptn = tab.getAttribute("cptn");
    var dtn = tab.getAttribute("dptn");
    var img = tab.getElementsByTagName("img");
    var inp = tab.getElementsByTagName("input");
    var siz = tab.getAttribute("siz");

    tab.setAttribute("cptn",dtn);

    if( dtn == "0" ) {
        if( ptn == "2" ) {
            inp[0].removeAttribute("name");
            inp[0].removeAttribute("value");
            img[0].src = this.getImage(id,0,siz);
        }
     }
     else if( dtn == "2" ) {
        if( ptn == "0" ) {
            inp[0].setAttribute("name",tab.getAttribute("name"));
            inp[0].setAttribute("value",tab.getAttribute("value"));
            img[0].src = this.getImage(id,2,siz);
        }
    }
}

apeos.util.toggleFindGroup = function(id,name,cur)
{
    var tab = document.getElementsByTagName("table");
    for(var i=0; i<tab.length; i++) {
        if(cur == tab[i]) {
            this.toggleSelect(id,cur,true);
        }
        else if(name == tab[i].getAttribute("name")) {
            this.toggleSelect(id,tab[i],false);
        }
    }
}

apeos.util.execHandler = function(body)
{
    if( body != null && body != "" ) {
        eval(body);
    }
}

apeos.util.toggleSelect = function(id,tab,on)
{
    var ptn = tab.getAttribute("cptn");
    var img = tab.getElementsByTagName("img");
    var inp = tab.getElementsByTagName("input");
    var siz = tab.getAttribute("siz");
    var onc = tab.getAttribute("onchg");
    var onu = tab.getAttribute("onup");
    var onl = tab.getAttribute("onck");

    if( ptn != "3" && ptn != "4") {
    if( on == true ) {
        if(ptn == "0") {
            tab.setAttribute("cptn","2");
            inp[0].setAttribute("name",tab.getAttribute("name"));
            inp[0].setAttribute("value",tab.getAttribute("value"));
            this.execHandler(onc);
        }
        img[0].src = this.getImage(id,2,siz);
        this.execHandler(onu);
        this.execHandler(onl);
    }
    else {
        if(ptn == "2") {
            tab.setAttribute("cptn","0");
            inp[0].removeAttribute("name");
            inp[0].removeAttribute("value");
            this.execHandler(onc);
        }
        img[0].src = this.getImage(id,0,siz);
    }
    }
}

apeos.util.toggleChange = function(id,tab)
{
    var ptn = tab.getAttribute("cptn");
    var img = tab.getElementsByTagName("img");
    var inp = tab.getElementsByTagName("input");
    var siz = tab.getAttribute("siz");
    var onc = tab.getAttribute("onchg");
    var onu = tab.getAttribute("onup");
    var onl = tab.getAttribute("onck");

    if( ptn != "3" && ptn != "4") {
        if( ptn == "0" ) { // off
            tab.setAttribute("cptn","2");
            inp[0].setAttribute("name",tab.getAttribute("name"));
            inp[0].setAttribute("value",tab.getAttribute("value"));
            img[0].src = this.getImage(id,2,siz);
            this.execHandler(onc);
        }
        else if( ptn == "2" ) { // on
            tab.setAttribute("cptn","0");
            inp[0].removeAttribute("name");
            inp[0].removeAttribute("value");
            img[0].src = this.getImage(id,0,siz);
            this.execHandler(onc);
        }
        this.execHandler(onu);
        this.execHandler(onl);
    }
}

apeos.util.getLable = function(label)
{
    var ary = label.split("&#8232;");
    var str = ary.join("<BR>");
    return str;
}


apeos.util.cntLabel = function(label)
{
    var ary = label.split("&#8232;");
    return (ary.length-1);
}

apeos.util.getLabelWidth = function(label)
{
    var ary = label.split("&#8232;");
    var i,max;
    for( i = max = 0; i < ary.length ; i++ ) {
      if( max < ary[i].length ) {
        max = ary[i].length;
      }
    }
    return max;
}

apeos.util.getText = function(wid,val)
{
    if( wid == 3 ) { /* text */
        return val;
    }
    var pwt = "";
    for (var i=0; i<val.length; i++) {
        pwt += "*";
    }
    return pwt;
}

apeos.util.chkText = function(val,max)
{
    if( max > 0 ) {
        val = val.slice(0,max);
    }
    var txt = "";
    for (var i=0; i<val.length; i++) {
        var n = val.charCodeAt(i);
        if( n >= 32 && n <= 126 ) {
            txt += val.charAt(i);
        }
        else {
            txt += "?";
        }
    }
    return txt;
}

apeos.util.commandGenerate  = function(wid,size,fid,id,name,val,tp,dis,evt)
{
    var prm = "";

    if( name == "" ) {
        this.setErrMsg(wid,"Invalid [name] argument");
        return true;
    }
    if( tp == "" ) {
        tp = "button";
    }
    if( val == "" ) {
        val = tp;
    }

    if( id ) {
        prm += " id=" + id;
    }

    if( size < 1 ) {
        size = 1;
    }
    if( size > 5 ) {
        size = 5;
    }
    prm += " siz=" + size;

    if( tp ) {
        prm += " btp=" + tp;
    }
    if( fid ) {
        prm += " fid=" + fid;
    }

    var str = this.getLable(val);

    if(!dis) {
        prm += ' name="' + name + '" value="' + val + '"';
    }

    var ptn = this.getPattern(false,dis);
    prm += " cptn=" + ptn;
    var tdw = ( this.cfont[size] * apeos.util.getLabelWidth(val) / 2 );
    var col = 'style="color:' + this.fcolor[ptn] + ';font-size:'+this.cfont[size]+'px;margin-top:'+this.cmarg[size]+';width:'+tdw+'px"';
    var evp = this.copyEvent(evt,wid,"apeos.util.commandMousedown","apeos.util.commandMouseup");
    var chg = this.csize[size] + (apeos.util.cntLabel(val)*this.cfont[size]);
    var css = new Array(9);

    for( var i=0; i < 9; i++ ) {
        switch(i) {
        case 4: 
            css[i] = 'style="background-color:'+this.bcolor[ptn]+'; height: '+chg+'px;width:'+tdw+'"';
            break;
        case 3:
        case 5: 
            css[i] = 'style="background-image: url('+this.getImage(2,ptn,i+1)+'); height: '+this.csize[size]+'px"';
            break;
        default:
            css[i] = 'style="background-image: url('+this.getImage(2,ptn,i+1)+')"';
        }
   }
    apeos.util.write('<table class="xrx-command" ' + prm + evp+'><tr><td class="tleft" '+css[0]+'></td><td class="tcenter" '+css[1]+'></td><td class="tright" '+css[2]+'></td></tr><tr><td class="mleft" '+css[3]+'></td><td class="mcenter" '+css[4]+'><input type="hidden"><div class="label" '+col+'>'+str+'</div></td><td class="mright" '+css[5]+'></td></tr><tr><td class="bleft" '+css[6]+'></td><td class="bcenter" '+css[7]+'></td><td class="bright" '+css[8]+'></td></tr></table>');
}

apeos.util.commandMousedown = function(id)
{
    this.pressing = true;
    var tab = event.currentTarget;
    var ptn = tab.getAttribute("cptn");
    var ond = tab.getAttribute("ondown");
    if( ptn == "0" ) {
        var tds = tab.getElementsByTagName("td");
        for( var i=0; i < tds.length; i++ ) {
            if( i != 4 ) { /* not center cell */
                tds[i].style.backgroundImage = 'url('+this.getImage(id,1,i+1)+')';
            }
            else {
                tds[i].style.backgroundColor = this.bcolor[1]
            }
        }
        this.execHandler(ond);
        this.beep(0);
    }
};
apeos.util.commandMouseup = function(id)
{
    if( this.pressing ) {
    var tab = event.currentTarget;
    var ptn = tab.getAttribute("cptn");
    var fid = tab.getAttribute("fid");
    var btp = tab.getAttribute("btp");
    var onu = tab.getAttribute("onup");
    var onl = tab.getAttribute("onck");

    if( ptn == "0" ) {
        var tds = tab.getElementsByTagName("td");
        for( var i=0; i < tds.length; i++ ) {
            if( i != 4 ) { /* not center cell */
                tds[i].style.backgroundImage = 'url('+this.getImage(id,0,i+1)+')';
            }
            else {
                tds[i].style.backgroundColor = this.bcolor[0]
            }
        }
        this.execHandler(onu);
        this.execHandler(onl);
    }
    if( fid ) {
        if( btp == "submit" ) {
        var inp = tab.getElementsByTagName("input");
        inp[0].setAttribute("name",tab.getAttribute("name"));
        inp[0].setAttribute("value",tab.getAttribute("value"));
        var frm = document.getElementById(fid);
        frm.submit();

	/* prevent ANT issue */

        var smt = frm.getAttribute("onsubmit");
        eval(smt);

        }
        if( btp == "reset" ) {
        var frm = document.getElementById(fid);
        var ret = frm.reset();

	/* prevent ANT issue */

        var rst = frm.getAttribute("onrst");
        eval(rst);

        }
    }
    this.pressing = false;
    }
};

apeos.util.textMouseup = function(id)
{
    if( this.pressing ) {
    var tab = event.currentTarget;
    var ptn = tab.getAttribute("cptn");
    var fid = tab.getAttribute("fid");
    var onu = tab.getAttribute("onup");
    var onl = tab.getAttribute("onck");

    if( ptn == "0" ) {
        this.addBlurListner(id,tab);
        var tds = tab.getElementsByTagName("td");
        for( var i=0; i < tds.length; i++ ) {
            if( i != 4 ) { /* not center cell */
                tds[i].style.backgroundImage = 'url('+this.getImage(id,4,i+1)+')';
            }
            else {
                tds[i].style.backgroundColor = this.bcolor[4]
            }
        }
        this.execHandler(onu);
        this.execHandler(onl);
    }
    this.pressing = false;
    }
};

apeos.util.textKeyPress = function(id,tab,code)
{
    var ptn = tab.getAttribute("cptn");
    var fid = tab.getAttribute("fid");
    var roy = tab.getAttribute("red");
    var onp = tab.getAttribute("onkpress");

    if( tab.pos == undefined ) {
        tab.pos = 0;
    }

    if( ptn == "0" && roy == undefined ) {

        var inp = tab.getElementsByTagName("input");
        var div = tab.getElementsByTagName("div");
        var siz = parseInt(tab.getAttribute("siz"));
        var max = parseInt(tab.getAttribute("max"));
        var str = "";
        var len = 0;

        if( inp[0].value ) {
            str = inp[0].value;
            len = str.length;
        }

        if( max == 0 || len < max ) {
            str = this.textKeyCode(code,str);
            inp[0].value = str;
            if( code == 8 ) {
                if( str.length >= siz ) {
                    tab.pos--;
                }
            }
            else if( str.length > siz ) {
                tab.pos++;
            }
            str = str.slice(tab.pos,tab.pos+siz);
            div[0].innerText = this.getText(id,str);
        }
        else if( len == max && code == 8 ) {
            str = this.textKeyCode(code,str);
            inp[0].value = str;
            if( str.length >= siz ) {
                tab.pos--;
            }
            str = str.slice(tab.pos,tab.pos+siz);
            div[0].innerText = this.getText(id,str);
        }
        this.execHandler(onp);
        this.beep(0);
    }
};

apeos.util.textKeyCode = function(code,text)
{
    var str = text; /*  ***?  */

    switch(code) {
        case 35: /* # */
            str += '#';
            break;
        case 42: /* aster */
            str += '*';
            break;
        case 44: /* Pause */
            str += ',';
            break;
        case  8: /* C */
            if( str ) {
            if( str.length > 0 ) {
                str = str.slice(0,str.length-1);
            }
            }
            break;
        default:
            if( code >= 48 && code <= 57 ) {
                str += ( code - 48 );
            }
    }
    return str;
}

apeos.util.textBlur = function(id,tab)
{
    var ptn = tab.getAttribute("cptn");
    var fid = tab.getAttribute("fid");

    if( ptn == "0" ) {
        var tds = tab.getElementsByTagName("td");
        for( var i=0; i < tds.length; i++ ) {
            if( i != 4 ) { /* not center cell */
                tds[i].style.backgroundImage = 'url('+this.getImage(id,0,i+1)+')';
            }
            else {
                tds[i].style.backgroundColor = this.icolor[0]
            }
        }
        return true;
    }
    return false;
};

apeos.util.inputReset = function(id,name)
{
    var tab = document.getElementsByTagName("table");
    for(var i=0; i<tab.length; i++) {
        if(name == tab[i].getAttribute("name")) {
            this.inputInit(id,tab[i]);
        }
    }
}

apeos.util.inputInit = function(id,tab)
{
   var def = tab.getAttribute("dval");
   if(def != "" ) {
       var inp = tab.getElementsByTagName("input");
       var div = tab.getElementsByTagName("div");
       inp[0].value = def;
       div[0].innerText = this.getText(id,def);
   }
}

apeos.util.inputGenerate  = function(wid,type,fid,id,name,val,read,size,max,dis,evt)
{
    var prm = "";
    var hdn = "";
    var str = "";

    if( name == "" ) {
        this.setErrMsg(wid,"Invalid [name] argument");
        return true;
    }

    this.hookResetFunc(wid,fid,name,"apeos.util.inputReset");

    if( id ) {
        prm += " id=" + id;
    }
    if( read ) {
        prm += " red=only";
    }
    if( size <= 0 ) {
        size = 20;
    }
    if( max <= 0 ) {
        max = 0;
    }
    if( type < 1 || type > 5 ) {
        type = 1;
    }
    prm += " siz=" + size;
    prm += " max=" + max;

    if( val ) {
        if( size < val.length ) {
          val = val.substr(0,size);
        }
        val = this.chkText(val,max);
        str = this.getText(wid,val);
    }

    if(!dis) {
        prm += ' name="' + name + '" value="' + val + '"';
        hdn += ' name="' + name + '" value="' + val + '"';
        if( !read ) {
            prm += ' dval="' + val + '"';
        }
    }

    var ptn = this.getPattern(false,dis);
    prm += " cptn=" + ptn;

    var wth = ( this.cfont[type] * size / 1.8 );

    var col = 'style="color:' + this.fcolor[ptn] + ';font-size:'+this.cfont[type]+'px; width: '+wth+'px;margin-top:'+this.cmarg[type]+'px"';
    var evp = this.copyEvent(evt,wid,"apeos.util.commandMousedown","apeos.util.textMouseup");
    var css = new Array(9);

    for( var i=0; i < 9; i++ ) {
        switch(i) {
        case 4: 
            css[i] = 'style="background-color:'+this.icolor[ptn]+'; height: '+this.csize[type]+'px; width: '+wth+'px"';
            break;
        case 3:
        case 5: 
            css[i] = 'style="background-image: url('+this.getImage(3,ptn,i+1)+'); height: '+this.csize[type]+'px"';
            break;
        default:
            css[i] = 'style="background-image: url('+this.getImage(3,ptn,i+1)+')"';
        }
   }
    apeos.util.write('<table class="xrx-command" ' + prm + evp+'><tr><td class="tleft" '+css[0]+'></td><td class="tcenter" '+css[1]+'></td><td class="tright" '+css[2]+'></td></tr><tr><td class="mleft" '+css[3]+'></td><td class="mcenter" '+css[4]+'><input type="hidden" '+hdn+'><div class="label" '+col+'>'+str+'</div></td><td class="mright" '+css[5]+'></td></tr><tr><td class="bleft" '+css[6]+'></td><td class="bcenter" '+css[7]+'></td><td class="bright" '+css[8]+'></td></tr></table>');
}

apeos.util.getValue = function(name,value)
{
   var inp = document.getElementsByTagName("input");
    for(var i=0; i<inp.length; i++) {
        if(inp[i].type == "hidden") {
          if( name == inp[i].name ) {
            return inp[i].value;
          }
        }
    }
    return null;
}

/******************************************************************
 * object creation API
 ******************************************************************/

createRadio = function(size,fid,label,id,name,val,check,dis,evt)
{
    apeos.util.toggleGenerate(0,size,fid,label,id,name,val,check,dis,evt);
}
createCheckbox = function(size,fid,label,id,name,val,check,dis,evt)
{
    apeos.util.toggleGenerate(1,size,fid,label,id,name,val,check,dis,evt);
}

createCommand = function(size,fid,id,name,val,tp,dis,evt)
{
    apeos.util.commandGenerate(2,size,fid,id,name,val,tp,dis,evt)
}

createInputText = function(type,fid,id,name,val,read,size,max,dis,evt)
{
    apeos.util.inputGenerate(3,type,fid,id,name,val,read,size,max,dis,evt);
}

createInputPassword = function(type,fid,id,name,val,read,size,max,dis,evt)
{
    apeos.util.inputGenerate(4,type,fid,id,name,val,read,size,max,dis,evt);
}

/******************************************************************
 * apeos object functions
 ******************************************************************/

apeos.docMouseUp = function(evt)
{
    apeos.util.checkBlur(evt.target);
    //apeos.util.print(false,"mu");
}

apeos.docKeyPress = function(evt)
{
    if( apeos.util.curtxt != null ) {
        apeos.util.textKeyPress(apeos.util.curwid,apeos.util.curtxt,evt.which)
    }
    apeos.util.print(false,"key");
    //document.body.focus();
}

apeos.onload = function() 
{
    //var nodes = document.body;
    //apeos.util.printDomTree(nodes, 1);

    apeos.util.showLog();

    apeos.util.onload = true;

    document.addEventListener("mouseup", apeos.docMouseUp, false);
    document.addEventListener("keypress", apeos.docKeyPress, false);


    //window.focus();
};

window.onload = apeos.onload;



