/*
 * (c) Copyright 1999-2015 PaperCut Software International Pty. Ltd.
 *
 * MFD related utilities
 */
"use strict";
var apeosSetup = false;
var ewbVersion = apeos.getBrowserVersion();
if (ewbVersion === "4") {
  // Setup Apeos library if running under EWBv4, has to be done before window.onload
  apeos.setupLibrary("DependLib/");
  apeosSetup = true;
}

// Set Date.now() => new Date.getTime() if it's not supported
if (!Date.now) {
  Date.now = function now() {
    return new Date().getTime();
  };
}

// Tracks current/last card insertion status reported by the framework
var cardInserted = false;
// Tracks current/last card insertion time
var cardInsertedTime = 0;

/**
 * Logs status message to our debug console if enabled
 */
function status(message) {
  if (consoleEnabled) {
   console.log(message);
  }
}

/**
 * Entry point
 */
window.onload = function() {
  // Pass through to onload in apeos library if in use
  apeosSetup && apeos.onload();
  BrowserExt.Init();
  status("onload");
  // TODO: not localized
  DIALOG.init({
    OK : "Ok",
    CONFIRM : "Yes",
    CANCEL : "No"
  }, {
    DISABLE_OVERLAY : false,
    BACKGROUND_COLOR : "#f5f5f5"
  });

  // Try to initialise and continue the startup once completed
  MFD.init({
      url : url,
      connectionTimeout : connectionTimeout,
      deviceVendor : deviceVendor,
      deviceModel : deviceModel,
      deviceGeneration : deviceGeneration,
      browserVersion : ewbVersion
    },
    function(resp) {
      MFD.logEvent("*** window.onload [auth] ***" +
        "\nEWB Version: " + ewbVersion +
        "\nUser Agent: " + window.navigator.userAgent +
        "\nPlatform: " + window.navigator.platform +
        "\nLanguage: " + window.navigator.language +
        "\nScreen: " + screen.width + "x" + screen.height + ", colorDepth: " + screen.colorDepth +
        "\nLCD Type: " + (apeos.getLcdType() === 0 ? "SVGA (800x600)" : "WVGA (800x480)") +
        "\nMemory Alloc: " + apeos.getAllocatedMemorySize() + " bytes" +
        "\nMemory Free: " + apeos.getFreeMemorySize() + " bytes" +
        "\nServer URL: " + url +
        "\nTimeout: " + connectionTimeout);

      // Set up display size and transition to login screen
      var screenNode = document.getElementById('auth_ui');
      screenNode.style.width = screen.width;
      screenNode.style.height = screen.height;
      MFD.changeScreen(screenNode.id);
    },
    function(message) {
      DIALOG.showMessageConfirm('Unexpected error during startup: ' + message);
    });
}

/**
 * Local UI hook triggered whenever hosted page (embedded UI) changes URL
 */
function setHostedPage(url) {
  url = url && url.replace(/password=.*(&)?/gi, 'password=<NOT SHOWN>');
  status("setHostedPage: " + url);
  MFD.logEvent("setHostedPage: " + url);
}

/**
 * Local UI hook invoked on every hosted page to sync the current/in-progress user
 */
function setUserLoggingOn(username) {
  tentativeUser = username || null;
}

/**
 * Local UI hook invoked to close application directly without logging-in the user
 */
function closeAuth() {
  status("closeAuth: user=" + loggedInUser);
  MFD.logEvent("closeAuth: user=" + loggedInUser);
  // Always go back to services home, since this was the only place where this app event is invoked from
  closeApp('allservice');
}

/**
 * Local UI hook that receives notifications of user hitting logout button in our UI
 */
function logout(reason) {
  // log out the user from the device if currently logged in
  if (tentativeUser) {
    MFD.loginReset();
    MFD.logEvent("logout: tentativeUser=" + tentativeUser + ", reason=" + reason);
    status("logout: tentativeUser=" + tentativeUser);
    tentativeUser = null;
  }

  if (loggedInUser) {
    MFD.logEvent("logout: loggedInUser=" + loggedInUser + ", reason=" + reason);
    status("logout: loggedInUser=" + loggedInUser);
    MFD.logout(
      function(resp) {
        // check ok==true
        loggedInUser = null;
        BrowserExt.SetScreenChange();
      },
      function(message) {
        DIALOG.showMessageConfirm('Logout failed: ' + message + "\nPlease logout using the panel hard key.",
        function() {
         logout(reason);
        });
      });
  }
}

/**
 * Local UI hook to inject pre-authenticated user session data as JSON string
 */
function setUserSessionInfo(userSession) {
  // Stop polling for card swipes as we're about to log the user in
  cardCheckTimer && clearTimeout(cardCheckTimer);
  cardCheckTimer = null;

  status("setUserSessionInfo: " + userSession);
  MFD.logEvent("setUserSessionInfo: " + userSession);
  if (userSession) {
    //its a hack for gen 9 and gen 10 device. Somehow it works this way.
    if (deviceGeneration < 10) {
      authenticateGen9(userSession);
    } else {
      authenticate(userSession);
    }
  }
}

/**
 * Authenticate the user to the device
 */
function authenticate(userSession) {
  MFD.logEvent("Try authenticating with Gen 10+");
  showAuthenticateWaitDialog();

  // Proceed with authentication immediately if card insertion is not currently active
  // or keep trying until maximum of configured cardHoldTime is elapsed
  if (cardInserted && (Date.now() - cardInsertedTime) < cardHoldTime) {
    MFD.logEvent("Retry authenticate with Gen 10+");
    setTimeout(function() { authenticate(userSession); }, 1000);
    return;
  }

  mfdAuthenticate(userSession);
}

/**
 * Authenticate the user to the device
 */
function authenticateGen9(userSession) {
  MFD.logEvent("Try authenticating with Gen 9");
  showAuthenticateWaitDialog();

  // Proceed with authentication immediately if card insertion is not currently active
  // or call mfd authenticate after cardHoldTime is elapsed
  if (cardInserted && (!isUseCardHoldTimeForGen9() || Date.now() - cardInsertedTime < cardHoldTime)) {
    MFD.logEvent("Retry authenticate with Gen 9");
    setTimeout(function () { authenticateGen9(userSession); }, 1000);
  } else {
    // its a hack for gen 9 device to force a delay to ensure we wait for cardHoldTime before unlocking the device
    setTimeout(function () { mfdAuthenticate(userSession); }, cardHoldTime);
  }
  //noinspection UnnecessaryReturnStatementJS - don't remove gen 9 device behaves weird
  return;
}

/**
 * The switch to turn using CardHoldTime for Gen9 ON/OFF is to set the final digit to 1
 *
 * Usually, CardHoldTime is only used for Gen10+ devices. A hotfix was added to allow using it for Gen9 devices.
 * This was a hotfix added for a customer who was experiencing a hung login screen when using Gen9 devices.
 * The cause of this bug we couldn't precisely narrow down, however the solution of using CardHoldTime appeared to work.
 * Zendesk: https://papercut.zendesk.com/agent/tickets/717032
 */
function isUseCardHoldTimeForGen9() {
  return (cardHoldTime.toString().slice(-1) === '1'); // Check the final digit is '1'
}

function showAuthenticateWaitDialog() {
  DIALOG.showMessageDialog('Please wait...', {COLOR : 'black', DISABLE_OVERLAY: true});
}

// Perform device authentication
function mfdAuthenticate(userSession) {
  userSession = JSON.stringify(JSON.parse(userSession));
  var param = "appName=" + XCP.APP_NAME + "&" + XCP.addParam("userSession", userSession);
  MFD.authenticate(param, function (result) { onUserAuthenticated(result); }, onUserAuthenticateError);
}

/**
 * User authentication request callback
 * @param result
 */
function onUserAuthenticated(result) {
  MFD.logEvent("onUserAuthenticated: " + JSON.stringify(result));
  if (result.ok) {
    loggedInUser = result.userInfo.userId;
    MFD.changeScreen('end');
  } else {
    // Login rejected
    onUserAuthenticateError(JSON.stringify(result));
  }
}

/**
 * User authentication request error callback
 * @param message
 */
function onUserAuthenticateError(message) {
  MFD.logEvent("onUserAuthenticateError: " + message);
  DIALOG.showMessageConfirm('Login failed.\nResult: ' + message, function() {
   returnToStartOnError && MFD.changeScreen("auth_ui");
 });
}

/**
 * Transition to start of login screen
 */
function auth_ui_function() {
  // Lock panel keys
  lockInput && BrowserExt.SetScreenChange("uilock:ON");

  // Initialise state
  init();
}

/**
 * Prepare URL for login to the app server using card number
 */
function cardLoginUrl(cardNo) {
   // URL encode actual card data as cards can include characters that need escaping,
   // eg. track separators/sentinels with MagTek reader and multi track cards.
  return "/login?cardNo=" + encodeURIComponent(cardNo) + "&time=" + Date.now();
}

/**
 * Initialise application state
 */
function init() {
  DIALOG.showMessageDialog('Initialising...', {COLOR : 'black'});

  cardInserted = false;
  cardInsertedTime = 0;
  tentativeUser = null;

  // Check if there is already an underlying user logged in to the device.
  // This is a user that has unlocked the device completely and logged into XCP platform.
  MFD.getLoggedOnUser(
    function(resp) {
      var initialUrl = '/init';
      var loggedInCard = null;

      MFD.logEvent("Logged on user state: " + JSON.stringify(resp));

      if (resp.results && resp.results.result === 'true') {
        loggedInUser = resp.results.username;
        loggedInCard = resp.results.cardNumber;

        if (loggedInUser) {
          if (resp.results.inProgress === 'true') {
            // Workaround for browser randomly reloading during unlock
            // If login is currently in progress of being completed ignore browser startup and just hide it
            MFD.logEvent("Ignore app startup while a user logon is still in progress: " + loggedInUser);
            closeApp();
            return;
          } else {
            // Confirmed that another login is currently active,
            // proceed directly to account confirmation where
            // they can either return back to device or logout
            MFD.logEvent("Another user is already logged in: " + loggedInUser);
            initialUrl = '/selectaccount?deviceUser=' + loggedInUser;
          }
        } else if (loggedInCard) {
          // Workaround for device randomly reloading the browser after card swipe causing
          // previous state (card swipe) to be lost.
          // If there is cached card number use it right away
          MFD.logEvent("Start up while previous card number is active: " + loggedInCard);
          initialUrl = cardLoginUrl(loggedInCard);
        }
      }
      MFD.logEvent("loggedInUser: " + loggedInUser +
        "\nloggedInCard: " + loggedInCard +
        "\nURL: " + initialUrl);

      // Check connection to the server URL
      MFD.checkServerStatus(initialUrl, function(resp) {
        if (initialUrl === '/init') {
          // Fetch additional server config on startup to be made available to the embedded plugins
          MFD.getServerConfiguration(url + "/config", function(resp) {
            MFD.logEvent("getServerConfiguration: " + JSON.stringify(resp));
          });
        }

        // Begin card checking once connection is established but wait for reader to stabilize due to hold time.
        // If a read was not in progress during startup (due to browser reload) then begin first card check as soon as possible.
        var cardReaderMonitoringDelay = loggedInCard ? cardHoldTime : 0;
        MFD.logEvent("Starting card reader monitoring in: " + cardReaderMonitoringDelay + "ms");
        pollStatus(cardReaderMonitoringDelay);
        DIALOG.closeMessageDialog();
      });
    },
    function(message) {
      DIALOG.showMessageConfirm('Unable to check device user status.\nReason: ' + message,
        function() {
          // retry init sequence
          init();
        });
    });
}

/**
 * Poll card reader status
 */
function getStatus() {
  if (cardReadEnabled) {
    if (cardCheckTimer) {
      clearTimeout(cardCheckTimer);
      cardCheckTimer = null;
    }
    MFD.getStatus(function(resp) { onGetStatusDone(resp) }, onGetStatusFail);
  } else {
    MFD.logEvent("Card reader monitoring is disabled");
  }
}

/**
 * Schedule card reader status polling
 */
function pollStatus(timeout) {
  cardCheckTimer && clearTimeout(cardCheckTimer);
  cardCheckTimer = setTimeout(function() { getStatus(); }, timeout || cardPollTimeout)
}

/**
 * Handles card reader status updates
 */
function onGetStatusDone(resp) {
  var deviceStatus = MFD.getDeviceStatus();
  if (debugEnabled && deviceStatus && deviceStatus.readerStatus) {
    var readerStatus = document.getElementById('reader_status');
    readerStatus.innerHTML = 'Reader: ' + deviceStatus.readerStatus.deviceName || deviceStatus.readerStatus.status;
  }
  if (debugEnabled && deviceStatus && deviceStatus.icCardStatus) {
    var cardStatus = document.getElementById('card_status');
    cardStatus.innerHTML = 'Card: ' + (deviceStatus.icCardStatus.icCardId + deviceStatus.serialNumber) || deviceStatus.icCardStatus.status;
  }

  var insertionEvent = deviceStatus.icCardStatus && deviceStatus.icCardStatus.status == "inserted-verified-pin" ||
                           deviceStatus.icCardStatus && deviceStatus.icCardStatus.status == "inserted";

  // Handle first insertion/removal state changes
  if (insertionEvent && !cardInserted) {
    debugEnabled && DIALOG.showMessageDialog('Card inserted', {COLOR : 'black', DISABLE_OVERLAY: true});
    MFD.logEvent("onGetStatusDone: " + JSON.stringify(resp));
    debugEnabled && alert('read card: ' + JSON.stringify(deviceStatus.icCardStatus));

    if (!deviceStatus.icCardStatus.icCardId) {
      // If card is inserted the card number must be communicated by our plugin as well
      DIALOG.showMessageConfirm('This card is not supported.' +
        '\nPlease consult the system administrator.' +
        '\nError on card reader: ' + deviceStatus.readerStatus.deviceName, function() {
        pollStatus(1000);
      });
      return;
    }

    // Determine how we need to respond to the swipe
    var cardNo = deviceStatus.icCardStatus.icCardId + deviceStatus.icCardStatus.serialNumber.substr(1);
    MFD.logEvent("Received card number of: " + cardNo);
    var loginUrl = null;

    cardInsertedTime = Date.now();

    if (tentativeUser) {
      // logout the user if one is logged on by card already
      logout("cardLogout");
      loginUrl = url + "/init";
    } else {
      // card swipe, transition to login via card screen
      loginUrl = url + cardLoginUrl(cardNo);
    }

    MFD.logEvent("Card insertion: " + cardNo + ", Change URL: " + loginUrl);
    loginByCard(loginUrl);
  } else if (MFD.getScreen() === 'auth_ui') {
    if (!insertionEvent && cardInserted) {
      // removal signalled: our plugin "holds" the current card for a period of time in memory to
      // ensure XCP framework sees it reliably, at this point the hold is expired and we see insertion -> removal state
      MFD.logEvent("Card removal");
      debugEnabled && DIALOG.showMessageDialog('Card removal', {COLOR : 'black', DISABLE_OVERLAY: true});
    }
    pollStatus();
  }

  // keep track of current insertion state
  cardInserted = insertionEvent;
  deviceStatus = null;
}

function loginByCard(loginUrl) {
  debugEnabled && DIALOG.showMessageDialog('Card inserted, Please Wait...', {COLOR : 'black', DISABLE_OVERLAY: true});
  var frame = document.getElementById('frame');
  status("change url from: " + frame.src + " to: " + loginUrl);
  frame.src = loginUrl;
  frame = null;
  pollStatus(1000 /* resume card poll after a short delay, while transitioning screen */);
}

/**
 * Handles card reader status check failure
 */
function onGetStatusFail(message) {
  MFD.logEvent("onGetStatusFail: " + message);
  if (message && /1008-reader not exist/.test(message)) {
    // ignore reader removal and don't treat as an error
    pollStatus(cardPollFailTimeout);
    return;
  }
  // Card reader on device not operational
  DIALOG.showMessageConfirm('Unable to connect to card reader.\nReason: ' + message, function () {
    // retry card poll a little later
    pollStatus(cardPollFailTimeout);
  });
}

/**
 * Transition to native screen
 */
function end_function() {
  closeApp();
}

/**
 * Close this application and transition to predefined native screen
 */
function closeApp(destinationScreen) {
  // hide embedded UI
  var frame = document.getElementById('frame');
  frame.style.display = 'none';

  cardCheckTimer && clearTimeout(cardCheckTimer);
  lockInput && BrowserExt.SetScreenChange("uilock:OFF");

  // Transition to appropriate native screen
  // See EWB External Specification section 3.6
  // menu, allservice, copy, copy_easy, box, scan_mbox, scan_email, scan_it, scan_pc, scan_url, scan_media...
  var screenName = MFD.mapScreenName(destinationScreen || initialScreen);
  MFD.logEvent("closeApp: setScreenChange=" + screenName);
  apeos.setScreenChange("menuto:" + screenName);
}
