/*
 * (c) Copyright 1999-2014 PaperCut Software International Pty. Ltd.
 *
 * Dialog utilities
 */
'use strict';
var DIALOG = function () {
  var LABEL_OK;
  var LABEL_YES;
  var LABEL_NO;
  var globalOpts;

  var INPUT_STYLE = 'height:32px';

  function createElement(type, id, className) {
    var el = document.createElement(type);
    el.id = id;
    if (className) {
      el.className = className;
    }
    return el;
  }

  function createInput(type, id, className, value, style) {
    var input = createElement('input', id, className);
    input.type = type;
    input.value = value;
    input.className = className;
    input.style = style;
    return input;
  }

  var modes = {
    OK: 0,
    PROMPT: 1,
    MESSAGE: 2
  };

  function showMessage(mode, message, callback, opts) {
    var dialog = document.getElementById('dialog');
    if (!dialog) {
      // Build the dialog content
      dialog = createElement('div', 'dialog', 'dialog');
      dialog.style.visibility = 'hidden';
      dialog.style.verticalAlign = 'middle';
      document.body.appendChild(dialog);

      var messageContainer = createElement('div', 'messageContainer');
      messageContainer.align = 'center';
      dialog.appendChild(messageContainer);

      var dialogmessage = createElement('p', 'dialogmessage');
      messageContainer.appendChild(dialogmessage);

      var okdialog = createElement('div', 'okdialog');
      okdialog.appendChild(createInput('button', 'okbutton', 'small-button', LABEL_OK, INPUT_STYLE));
      dialog.appendChild(okdialog);

      var yesnodialog = createElement('div', 'yesnodialog');
      yesnodialog.appendChild(createInput('button', 'confirmbutton', 'small-button', LABEL_YES, INPUT_STYLE));
      yesnodialog.appendChild(createInput('button', 'cancelbutton', 'small-button', LABEL_NO, INPUT_STYLE));
      dialog.appendChild(yesnodialog);
    }
    dialog = null;

    var dialogOverlay = document.getElementById('dialog-overlay');
    if (!dialogOverlay) {
      document.body.appendChild(createElement('div', 'dialog-overlay', 'dialog-overlay'));
    }
    dialogOverlay = null;

    if (message) {
      if (opts && opts.DISABLE_OVERLAY) {
        document.getElementById('dialog-overlay').style.visibility = 'hidden';
      } else {
        document.getElementById('dialog-overlay').style.visibility = 'visible';
      }

      if (opts && opts.COLOR) {
        document.getElementById('dialog').style.color = opts.COLOR;
      }
      if (opts && opts.BACKGROUND_COLOR) {
        document.getElementById('dialog').style.backgroundColor = opts.BACKGROUND_COLOR;
      }
      document.getElementById('dialog').style.visibility = 'visible';
      // newlines to HTML break
      document.getElementById('dialogmessage').innerHTML = message.replace(/\n/g, '<br/>');

      // action handler shared by all buttons
      var eventHandler = function(event) {
        event.preventDefault();
        event.target.removeEventListener('click', eventHandler, false);

        // dismiss and invoke callback if any
        document.getElementById(event.target.id).blur();
        document.getElementById('dialog').style.visibility = 'hidden';
        document.getElementById('dialog-overlay').style.visibility = 'hidden';
        event.target.style.visibility = 'hidden';

        if (callback) {
          callback(event);
        }
      };

      if (mode === modes.OK) {
        // confirm mode, single ok button
        document.getElementById('okdialog').style.display = 'block';
        document.getElementById('yesnodialog').style.display = 'none';
        document.getElementById('okbutton').style.visibility = 'visible';
        document.getElementById('okbutton').focus();
        document.getElementById('okbutton').value = LABEL_OK;
        document.getElementById('okbutton').addEventListener('click', eventHandler, false);
      } else if (mode === modes.PROMPT) {
        // question mode, yes/no buttons
        document.getElementById('okdialog').style.display = 'none';
        document.getElementById('yesnodialog').style.display = 'block';
        document.getElementById('confirmbutton').style.visibility = 'visible';
        document.getElementById('confirmbutton').focus();
        document.getElementById('confirmbutton').addEventListener('click', eventHandler, false);
        document.getElementById('cancelbutton').addEventListener('click', eventHandler, false);
      } else {
        // message mode
        document.getElementById('okdialog').style.display = 'block';
        document.getElementById('yesnodialog').style.display = 'none';
        document.getElementById('okbutton').style.visibility = 'hidden';
      }
    } else {
      document.getElementById('dialog-overlay').style.visibility = 'hidden';
      document.getElementById('dialog').style.visibility = 'hidden';
    }
  }

  function copyProperties(src, dest) {
    if (src) {
      for (var p in src) {
        dest[p] = src[p];
      }
    }
    return dest;
  }

  function cloneObject(src) {
    var clone = {};
    return copyProperties(src, clone);
  }

  // return clone of one object with properties of another
  function cloneWithProperties(srcClone, srcProps) {
    return copyProperties(srcProps, cloneObject(srcClone));
  }

  return {
    options: {
      DISABLE_OVERLAY: false,
      COLOR: 'red',
      BACKGROUND_COLOR: 'white'
    },

    // TODO: integrate easy dialog type
    type: {
      ERROR: 0,
      WARN: 1,
      INFO: 2
    },

    /**
     * Initialise using localized data
     * @param labelData
     */
    init: function(labelData, opts) {
      if (!labelData) {
        // fallback to defaults
        labelData = {OK : 'Ok', CONFIRM : 'Yes', CANCEL : 'No'};
      }
      LABEL_OK = labelData.OK;
      LABEL_NO = labelData.CANCEL;
      LABEL_YES = labelData.CONFIRM;
      globalOpts = copyProperties(opts, this.options);
    },

    /**
     * Show or clear message confirmation dialog
     * @param message
     * @param callback
     */
    showMessageConfirm: function(message, callback, options) {
      // confirm mode
      return showMessage(modes.OK, message, callback, cloneWithProperties(globalOpts, options));
    },

    /**
     * Show or clear message prompt dialog
     * @param message
     * @param callback
     */
    showMessagePrompt: function(message, callback, options) {
      // prompt mode
      return showMessage(modes.PROMPT, message, callback, cloneWithProperties(globalOpts, options));
    },

    showMessageDialog: function(message, options) {
      return showMessage(modes.MESSAGE, message, null, cloneWithProperties(globalOpts, options));
    },

    closeMessageDialog: function() {
      showMessage(null, null);
    }
  };
}();