/*
 * (c) Copyright 1999-2014 PaperCut Software International Pty. Ltd.
 *
 * XCP related utilities
 */
"use strict";
var XCP = function () {
  var APP_NAME = "PaperCut_Auth";
  var XCP_URL = "http://localhost:8001/xcp/rpc/";
  // XIP: There is an undocumented limitation of 255 characters on all parameter values. 
  var MAX_PARAM_LEN = 255;

  if (typeof ActiveXObject == "function" && typeof XMLHttpRequest == "undefined") {
    XMLHttpRequest = function() {
      return new ActiveXObject("Microsoft.XMLHTTP");
    };
  }

  function defaultCallbackError(msg) {
    window.alert(msg);
  }
  
  function jsonParseLenient(json) {
    try {
      return JSON.parse(json);
    } catch (e) {
      return null;
    }
  }

  /**
   * Invoke XCP service
   */
  function callService(command, param, callbackSuccess, callbackError) {
    if (!callbackError) {
      callbackError = defaultCallbackError;
    }
    var startTime = new Date().getTime();
    var xhr = new XMLHttpRequest();
    xhr.callbackSuccess = callbackSuccess;
    xhr.callbackError = callbackError;
    xhr.onreadystatechange = function() {
      if (xhr.readyState == 4) {
        var elapsed = new Date().getTime() - startTime;
        consoleEnabled && status("callServiceDone (" + elapsed + "ms): " + command +
            "=> HTTP/" + xhr.status + ":" + xhr.responseText +
            "\nParams: " + param);

        if ((xhr.status == 200 || xhr.status == 500) && xhr.responseText !== null) {
          var json = jsonParseLenient(xhr.responseText);


          if (json) {
            if (json.errCode && json.errMsg) {
              xhr.callbackError("XCP operation '" + command + "' failed: " + json.errCode + "-" + json.errMsg + "\n" +
                "\nHTTP: " + xhr.status + ":" + xhr.responseText +
                "\nParams: " + param
              );
            } else {
              xhr.callbackSuccess(json);
            }
          } else {
            xhr.callbackError("Failed to parse JSON: " + xhr.responseText);
          }
        } else {
          xhr.callbackError("Error: status: " + xhr.status + " text: " + xhr.responseText);
        }
        xhr = null;
      }
    };
    consoleEnabled && status("callService: " + XCP_URL + command + ", payload: " + param);
    xhr.open("post", XCP_URL + command, true);
    xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    xhr.send(param);
  }

  /**
   * Add service parameter with name and value, breaking up long values into multiple chunks as needed
   */
  function addParam(name, value) {
    var param;
    var i, j, chunkSize = MAX_PARAM_LEN;

    if (value.length > MAX_PARAM_LEN) {
      // Add parameter in multiple chunks named: name0...nameN
      var chunk = 0;
      for (i = 0, j = value.length; i < j; i += chunkSize) {
        chunk = i / MAX_PARAM_LEN;
        var slice = value.slice(i, i + chunkSize);
        var paramVal = name + chunk + "=" + encodeURIComponent(slice);
        if (i === 0) {
          param = paramVal;
        } else {
          param += "&" + paramVal;
        }
      }
      // finally include the number of chunks required using the same name with a fixed suffix
      param += "&" + name + "Parts=" + (++chunk);
      return param;
    } else {
      // Add as is
      return name + "=" + value;
    }
  }

  return {
    XCP_URL: XCP_URL,
    APP_NAME: APP_NAME,
    callService: callService,
    addParam: addParam
  };
}();